import ast
import json
import os
import re
import sys
from functools import partial
from importlib import import_module
from optparse import make_option
from shutil import copyfile

from django.conf import settings
from django.core.management.base import BaseCommand
from django.db.models.fields import NOT_PROVIDED, DateField
from django.utils.encoding import force_unicode

import symmetric.management.overrides
from symmetric.functions import camel_case_to_underscore, underscore_to_camel_case
from symmetric.management.codeemitter import CodeEmitter
from symmetric.management.functions import get_base_classes, get_resource_type, get_subclass_filter, format_regex_stack, is_readonly, is_excluded, is_included
from symmetric.management.functions import get_model_name, get_model_name_plural, get_collection_name, get_collection_name_plural
from symmetric.management.rules import ApiFieldRule
from symmetric.management.translate import translate_code
from symmetric.models import get_related_model
from symmetric.views import ApiAction, ApiRequirement, BasicApiView, api_view

try:
	from generate import backbone_callback
except:
	def backbone_callback(name, model, collection, **kwargs):
		pass

class BackboneAttributeTransformer(ast.NodeTransformer):
	"""Any access to self.property_name should be converted to self.attributes.property_name if it is a field."""
	def __init__(self, model):
		self.model = model

	def visit_Attribute(self, node):
		if type(node.value) is ast.Name and node.value.id == 'self':
			if type(node.attr) is str and node.attr != 'id':
				try:
					self.model._meta.get_field(node.attr)
					node.attr = 'get("%s")' % node.attr
				except:
					pass
		else:
			self.visit(node.value)
		return node

def unjson(jsn):
	# Convert spaces to tabs and json keys to unquoted javascript attributes
	return re.sub(r'"([a-zA-Z0-9][^"]*)":', r'\1:', jsn)

separators = (',', ': ')

class Command(BaseCommand):
	help = 'Generate backbone models for API endpoints.'
	option_list = BaseCommand.option_list + (
			make_option('--dest',
				dest='dest',
				type='string',
				default='.',
				help='Output the models and colllections into models/ and collections/ inside the path given.'),
			make_option('--module-type',
				dest='module_type',
				type='string',
				default='global',
				help='Output the models inside the specified module type. Options are: global, amd, cjs, esm.'),
			make_option('--es6',
				action='store_true',
				dest='es6',
				default=False,
				help='Output the code as es6 code. If module-type is not given also implies esm module-type.'),
			make_option('--defaults',
				action='store_true',
				dest='defaults',
				default=False,
				help='Add in available default values into the models.'),
			make_option('--descriptions',
				action='store_true',
				dest='descriptions',
				default=False,
				help='Add the help_text for each field under a descriptions object.'),
			make_option('--titles',
				action='store_true',
				dest='titles',
				default=False,
				help='Add the verbose name for each field under a titles object.'),
			make_option('--choices',
				action='store_true',
				dest='choices',
				default=False,
				help='Add the choices for each field under a choices object.'),
			make_option('--formats',
				action='store_true',
				dest='formats',
				default=False,
				help='Add all field format messages used as errors when a field is invalid.'),
			make_option('--cord',
				action='store_true',
				dest='cord',
				default=False,
				help='Use cord functionality for defining properties, validation, and submodels.'),
			make_option('--validation',
				action='store_true',
				dest='validation',
				default=False,
				help='Add a validation method to each model.'),
			make_option('--validation-method',
				dest='validation_method',
				type='string',
				default='validate',
				help='Validation method to call, defaults to the global validate method. If --cord is given this default will become Backbone.Cord.validate'),
			make_option('--indent',
				dest='indent',
				type='int',
				default=0,
				help='Each tab should instead indent with this number of spaces.')
		)

	def add_choices(self, model):
		choices = {}
		for field in model._meta.fields:
			if is_excluded(model, field.name):
				continue
			if hasattr(field, 'choices') and field.choices:
				field_choices = {}
				for choice in field.choices:
					field_choices[choice[0]] = choice[1]
				key = field.name
				if self.camelcase:
					key = underscore_to_camel_case(key)
				choices[key] = field_choices
		if choices:
			self.models[get_model_name(model)]['choices'] = choices

	def add_defaults(self, model):
		name = get_model_name(model)
		for field in model._meta.fields:
			if is_excluded(model, field.name) or is_readonly(model, field.name):
				continue
			default = field.default
			if isinstance(default, type):
				default = default()
			if default is not NOT_PROVIDED and not callable(default):
				if not self.models[name].has_key('defaults'):
					self.models[name]['defaults'] = {}
				key = field.name
				if self.camelcase:
					key = underscore_to_camel_case(key)
				self.models[name]['defaults'][key] = default

	def add_descriptions(self, model):
		subtitles = {}
		for field in model._meta.fields:
			if is_excluded(model, field.name):
				continue
			if hasattr(field, 'help_text') and field.help_text:
				key = field.name
				if self.camelcase:
					key = underscore_to_camel_case(key)
				subtitles[key] = force_unicode(field.help_text)
		if subtitles:
			self.models[get_model_name(model)]['subtitles'] = subtitles

	def add_titles(self, model):
		titles = {}
		for field in model._meta.fields:
			if is_excluded(model, field.name):
				continue
			key = field.name
			if self.camelcase:
				key = underscore_to_camel_case(key)
			titles[key] = field.verbose_name.title()
		if titles:
			self.models[get_model_name(model)]['titles'] = titles

	def add_instructions(self, model):
		instructions = {}
		for field in model._meta.fields:
			if is_excluded(model, field.name):
				continue
			key = field.name
			if self.camelcase:
				key = underscore_to_camel_case(key)
			message = force_unicode(field.error_messages.get('invalid', field.error_messages.get('invalid_choice', '')))
			default_invalid = default_invalid_choice = None
			for cls in reversed([field.__class__] + get_base_classes(field.__class__)):
				if not hasattr(cls, 'default_error_messages'):
					continue
				default_invalid = cls.default_error_messages.get('invalid', default_invalid)
				default_invalid_choice = cls.default_error_messages.get('invalid_choice', default_invalid_choice)
			if message and message != default_invalid and message != default_invalid_choice:
				instructions[key] = message
		if instructions:
			self.models[get_model_name(model)]['instructions'] = instructions

	def add_validation(self, model):
		rules = {}
		for field in model._meta.fields:
			if is_excluded(model, field.name) or is_readonly(model, field.name):
				continue
			key = field.name
			if self.camelcase:
				key = underscore_to_camel_case(key)
			rules[key] = ApiFieldRule(field).rule
		if rules:
			self.models[get_model_name(model)]['rules'] = rules
			if not self.cord:
				self.models[get_model_name(model)]['validate'] = 'validate'

	def add_parse(self, model):
		fields = []
		for field in model._meta.fields:
			if is_excluded(model, field.name):
				continue
			# Parse DateField and DateTimeFields, but not TimeFields
			if isinstance(field, DateField):
				if self.camelcase:
					fields.append("'%s'" % underscore_to_camel_case(field.name))
				else:
					fields.append("'%s'" % field.name)
		if fields:
			self.models[get_model_name(model)]['parse'] = ', '.join(fields)

	def translate_properties(self, model):
		lang = 'es6' if self.es6 else 'js'
		model_properties = {}
		model_properties_args = {}
		for cls in [model] + get_base_classes(model):
			for name in cls.__dict__:
				attr = cls.__dict__[name]
				if type(attr) is property and attr.fget and hasattr(attr.fget, 'api_code'):
					if self.camelcase:
						name = underscore_to_camel_case(name)
					if getattr(attr.fget, 'api_translations', None) and attr.fget.api_translations.has_key(lang):
						model_properties[name] = attr.fget.api_translations[lang]
					else:
						model_properties[name] = translate_code(attr.fget.api_code, lang, BackboneAttributeTransformer(model))
		# If using Cord, then convert all references to other api properties to a get model call then to just arguments passed into the getter instead
		if self.cord:
			def replace_properties(m):
				prop = m.group(1)
				if prop in model_properties:
					return 'this.get("%s")' % prop
				return m.group(0)
			def replace_attributes(args, m):
				attr = m.group(1)
				args.add(attr)
				return attr
			for name, code in model_properties.items():
				model_properties_args[name] = set()
				if code.find('this.id'):
					model_properties_args[name].add('id')
				code = re.sub(r'this.([0-9a-zA-Z_]*)', replace_properties, code)
				model_properties[name] = re.sub(r'this.get\("([0-9a-zA-Z_]*)"\)', partial(replace_attributes, model_properties_args[name]), code)
		model_name = get_model_name(model)
		if model_properties:
			self.model_properties[model_name] = model_properties
		if model_properties_args:
			self.model_properties_args[model_name] = model_properties_args

	def get_include_related(self, model):
		related_models = {}
		name = get_model_name(model)
		if hasattr(model, 'API') and hasattr(model.API, 'include_related'):
			include_related = model.API.include_related
			for field in model._meta.fields:
				if field.name in include_related:
					if self.camelcase:
						field_name = underscore_to_camel_case(field.name)
					else:
						field_name = field.name
					related_models[field_name] = get_model_name(get_related_model(field))
		if related_models:
			self.include_related[name] = related_models

	def get_readonly_fields(self, model):
		fields = []
		included_fields = []
		name = get_model_name(model)
		for field in model._meta.fields:
			if is_excluded(model, field.name):
				continue
			if is_readonly(model, field.name):
				key = field.name
				if self.camelcase:
					key = underscore_to_camel_case(key)
				if is_included(model, field.name):
					included_fields.append(key)
				fields.append(key)
		if field:
			self.readonly_fields[name] = fields
		if included_fields:
			self.readonly_included_fields[name] = included_fields

	def add_model(self, model, extra):
		# Add the model for any resource type
		name = get_model_name(model)
		if not self.models.has_key(name):
			self.models[name] = {}
			if self.renameid:
				self.models[name]['idAttribute'] = camel_case_to_underscore(name) + '_id'
				if self.camelcase:
					self.models[name]['idAttribute'] = underscore_to_camel_case(self.models[name]['idAttribute'])
			self.add_parse(model)
			if self.choices:
				self.add_choices(model)
			if self.defaults:
				self.add_defaults(model)
			if self.descriptions:
				self.add_descriptions(model)
			if self.formats:
				self.add_instructions(model)
			if  self.titles:
				self.add_titles(model)
			if self.validation:
				self.add_validation(model)
			self.translate_properties(model)
			self.get_include_related(model)
			self.get_readonly_fields(model)
		self.models[name].update(extra)

	def get_model_replacement(self, view):
		subclass_filter = get_subclass_filter(view)
		if subclass_filter:
			model_str = '|'.join([get_model_name(model) for model in subclass_filter.subclasses])
		else:
			model_str = get_model_name(view.model)
		return model_str

	def get_model_unreplacement(self, match):
		models = match if isinstance(match, (str, unicode)) else match.group(1)
		models = models.split('|')
		count = len(models)
		if count > 1:
			models = ['if (attrs.hasOwnProperty(%s.prototype.idAttribute)) {\nreturn new %s(attrs, options);\n}\n' % (model, model) for model in models if self.models.has_key(model)]
			if len(models) < count:
				models.append('return new Backbone.Model(attrs, options);\n')
			return '%s {\noptions = options || {};\noptions.parse = true;\n%s}' % (self.get_anon_func('attrs, options'), 'else '.join(models))
		else:
			return models[0]

	def get_anon_func(self, args=''):
		if self.es6:
			return '(%s) =>' % args
		else:
			return 'function(%s)' % args

	def get_proto_func(self, name, args=''):
		if self.es6:
			return '%s(%s)' % (name, args)
		else:
			return '%s: function(%s)' % (name, args)

	def enum_patterns(self, patterns):
		for pattern in patterns:
			if pattern.callback:
				if isinstance(pattern.callback, (api_view, BasicApiView)):
					self.regex_stack.append(pattern._regex)
					view = pattern.callback
					url = format_regex_stack(self.regex_stack)
					resource_type = get_resource_type(self.regex_stack, pattern)
					model_extra = {}
					if resource_type == 'Collection':
						collection_name = get_collection_name(view)
						if url.find('<object_id>') == -1 and url.find('<slug>') == -1:
							# Unrelated
							self.collections[collection_name] = {'url': url, 'model': 'model###' + self.get_model_replacement(view)}
						else:
							# Related
							parent_name = get_model_name(view.parent_model)
							if url.find('<object_id>') != -1:
								if not self.related_collections.has_key(parent_name):
									self.related_collections[parent_name] = []
								self.related_collections[parent_name].append({'collection': collection_name, 'url': url.replace('<object_id>', '" + this.id + "')})
								# Generate a blank collection class for related items if needed
								if not self.collections.has_key(collection_name):
									self.collections[collection_name] = {'model': 'model###' + get_model_name(view.model)}
					elif resource_type == 'Object':
						if url.find('<object_id>') != -1:
							# Remove the last piece of the url path, should be <object_id>, don't look for <slug> because the idAttribute above only allows the object id
							model_extra['urlRoot'] = '/' + '/'.join(url.strip('/').split('/')[:-1])
					elif resource_type == 'Single Object':
						self.singletons[get_model_name(view.model)] = '/' + url.strip('/')
					# Add the model for any view without a subclass filter
					if not get_subclass_filter(view):
						self.add_model(view.model, model_extra)
					self.regex_stack.pop()
			else:
				self.regex_stack.append(pattern._regex)
				self.enum_patterns(pattern.url_patterns)
		if self.regex_stack:
			self.regex_stack.pop()

	def print_module_header(self, f=sys.stdout, dependencies=[]):
		# Module definition start
		deps = ['backbone', 'underscore'] + dependencies
		args = ['Backbone', '_'] + [dep.split('/')[-1] + ('Collection' if dep.startswith('../collections') else '') for dep in dependencies]
		if self.module_type == 'amd':
			deps = ["'%s'" % dd for dd in deps]
			deps = ', '.join(deps)
			args = ', '.join(args)
			self.emit("define([%s], %s {" % (deps, self.get_anon_func(args)))
		elif self.module_type != 'esm':
			self.emit('(%s {' % self.get_anon_func())
		if self.module_type != 'esm':
			self.emit('"use strict";', '')
		if self.module_type == 'cjs':
			for i in range(len(deps)):
				self.emit("%s %s = require('%s');" % (self.const, args[i], deps[i]))
			self.emit('')
		elif self.module_type == 'esm':
			for i in range(len(deps)):
				self.emit("import %s from '%s';" % (args[i], deps[i]))
			self.emit('')

	def print_module_footer(self, f=sys.stdout, name=None):
		# Module definition end
		if self.module_type == 'amd':
			if name:
				self.emit('return %s; });' % name)
			else:
				self.emit('return { models: models, collections: collections, singletons: singletons}; });')
		elif self.module_type == 'cjs':
			if name:
				self.emit('module.exports = %s;' % name)
			self.emit(
				'exports.models = models;',
				'exports.collections = collections;',
				'exports.singletons = singletons;'
			)
		elif self.module_type == 'esm':
			if name:
				self.emit('export default %s;' % name)
			else:
				self.emit('export default { models, collections, singletons };')
		else:
			if name:
				self.emit('window.%s = %s' % (name, name))
			else:
				self.emit(
					'window.models = models;',
					'window.collections = collections;',
					'window.singletons = singletons;'
				)
		if self.module_type != 'amd' and self.module_type != 'esm':
			self.emit('})();')

	def output_model(self, name, model):
		path = os.path.join(self.dest_models, '%s.js' % name)

		dependencies = []
		include_related = self.include_related.get(name)
		if include_related:
			dependencies += ['models/%s' % ir for ir in include_related.values()]
		model_properties = self.model_properties.get(name)
		if model_properties and self.cord:
			model['computed'] = 'computed'
		related_collections = self.related_collections.get(name)
		if related_collections:
			model['related'] = 'related'
			# dependencies += ['./%s' % r['model'] for r in related_collections]
			dependencies += ['../collections/%s' % r['collection'] for r in related_collections]
		readonly_fields = self.readonly_fields.get(name, [])
		readonly_included_fields = self.readonly_included_fields.get(name, [])
		if readonly_fields:
			model['toJSON'] = 'toJSON'

		backbone_callback(name, model, None)

		with open(path, 'w') as f:
			self.emit = CodeEmitter(f, self.indent)
			self.print_module_header(f, dependencies)
			model_js = unjson(json.dumps(model, separators=separators, indent=True, sort_keys=True))

			# Convert all urlRoot attributes into functions.
			# The collection's url should be overridden when the object already exists with an id (!isNew) because it may be part of a heterogeneous subclass collection or related collection that does not have an endpoint for updating the object
			# A newUrl is also defined/undefined when the model is created outside of a collection it can still use its collection's url for creating a new instance
			model_js = re.sub(r'urlRoot: "([^"]*)"',
				r"""%s {
					if (!this.isNew()) {
						return "\1";
					} else if (!this.collection) {
						return this.newUrl;
					}
				}""" % self.get_proto_func('urlRoot'), model_js)

			# Convert all parse attributes into a function
			parse_related = []
			if include_related:
				for i, field_name in enumerate(include_related):
					parse_related.append('if (response.%s) {\nresponse.%s = new %s(response.%s, { parse: true });\n}' % (field_name, field_name, include_related[field_name], field_name))
			parse_related = '\n'.join(parse_related)
			model_js = re.sub(r'parse: "([^"]*)"',
				r"""%s {
					if (response) {
						_.each([\1], %s {
							if (response[attr]) {
								response[attr] = new Date(response[attr]);
							}
						});
						%s
					}
					// Custom parse function can also process response
					if (this.extendedParse) {
						response = this.extendedParse(response);
					}
					return response;
				}""" % (self.get_proto_func('parse', 'response'), self.get_anon_func('attr'), parse_related), model_js)

			# Add to the toJSON function to exclude readonly fields
			readonly_fields = '\n'.join(['delete data.%s;' % ro for ro in readonly_fields])
			id_suffix = 'Id' if self.camelcase else '_id'
			readonly_included_fields = '\n'.join(["data.%s%s = this.get('%s').id;" % (included, id_suffix, included) for included in readonly_included_fields])
			if readonly_included_fields:
				readonly_included_fields = '\n' + readonly_included_fields
			model_js = re.sub(r'toJSON: "toJSON"',
				"""%s {
					%s data = Backbone.Model.prototype.toJSON.call(this);
					%s%s
					return data;
				}""" % (self.get_proto_func('toJSON'), self.const, readonly_fields, readonly_included_fields), model_js)

			# Convert all validate entries to functions
			model_js = re.sub(r'validate: "validate"',
				"""%s {
					%s attr, rule, ret, errors = {};
					for (attr in attributes) {
						if (attributes.hasOwnProperty(attr)) {
							rule = this.rules[attr];
							if (rule) {
								if (rule.equals === null && rule.equals === void(0)) {
									rule.equals = this.choices && this.choices[attr];
								}
								ret = %s(attributes[attr], rule);
								if (ret !== true) {
									errors[attr] = ret;
								}
							}
						}
					}
					// Custom validation can also add to the errors object
					if (this.extendedValidate) {
						this.extendedValidate(errors);
					}
					if (Object.keys(errors).length) {
						return errors;
					}
				}""" % (self.get_proto_func('validate', 'attributes'), self.local, self.validation_method), model_js)

			# Add on any computed api properties
			computed = []
			if model_properties:
				if self.cord:
					computed.append('computed: {')
					for i, prop_name in enumerate(model_properties):
						comma = '' if i == len(model_properties) - 1 else ','
						args = list(self.model_properties_args[name][prop_name])
						computed.extend((
							'%s: %s {' % (prop_name, self.get_anon_func(', '.join(args))),
								'if (%s === "__args__") {' % args[0],
									'return ["%s"];' % '", "'.join(args),
								'}',
								'return %s;' % model_properties[prop_name],
							'}%s' % comma
						))
					computed.append('}')
					model_js = re.sub(r'computed: "computed"', '\n'.join(computed), model_js)
					computed = None
				else:
					computed.extend(('Object.defineProperties(%s.prototype, {' % name))
					for i, prop_name in enumerate(model_properties):
						comma = '' if i == len(model_properties) - 1 else ','
						computed.extend((
							'%s: {' % prop_name,
								'get: %s {' % self.get_anon_func(),
									'return %s;' % model_properties[prop_name],
								'}',
							'}%s' % comma
						))
					computed.extend(('});', ''))

			get_related = []
			if related_collections:
				for i, related in enumerate(related_collections):
					comma = '' if i == len(related_collections) - 1 else ','
					get_related.extend((
						'%s {' % self.get_proto_func('get%sCollection' % related['collection']),
							'%s collection = new %sCollection();' % (self.const, related['collection']),
							'collection.url = "%s";' % related['url'],
							'return collection;',
						'}' + comma
					))
				model_js = re.sub(r'related: "related"', '\n'.join(get_related), model_js)

			self.emit(*('%s %s = Backbone.Model.extend(%s);\n' % (self.const, name, model_js)).split('\n'))

			# Non-cord computed properties
			if computed:
				self.emit(*model_properties)

			# If this model also has a singleton instance add a function to get it
			if self.singletons.has_key(name):
				model_js = self.emit(
					'%s sharedInstance;' % self.local,
						'%s.getShared%s = %s {' % (name, name, self.get_anon_func()),
						'if (!sharedInstance) {',
							'sharedInstance = new %s();' % name,
							'sharedInstance.url = "%s";' % self.singletons[name],
						'}',
						'return sharedInstance;',
					'};', '')

			self.print_module_footer(f, name)

	def output_collection(self, name, collection):
		path = os.path.join(self.dest_collections, '%s.js' % name)
		backbone_callback(name, None, collection)
		with open(path, 'w') as f:
			self.emit = CodeEmitter(f, self.indent)

			collection_js = unjson(json.dumps(collection, separators=separators, indent=4))
			# Convert the special model###modelName code into a models.Class or a function for a heterogeneous collection
			dependencies = ['../models/' + model for model in re.findall(r'"model###(.*)"', collection_js)[0].split('|')]
			collection_js = re.sub(r'"model###(.*)"', self.get_model_unreplacement, collection_js)
			name = name + 'Collection'
			self.print_module_header(f, dependencies)
			self.emit(*('%s %s = Backbone.Collection.extend(%s);\n' % (self.const, name, collection_js)).split('\n'))
			self.print_module_footer(f, name)

	def handle(self, *args, **options):
		self.dest = os.path.abspath(options['dest'])
		self.defaults = options['defaults']
		self.choices = options['choices']
		self.descriptions = options['descriptions']
		self.titles = options['titles']
		self.formats = options['formats']
		self.cord = options['cord']
		self.es6 = options['es6']
		self.module_type = options['module_type']
		self.validation = options['validation']
		self.validation_method = options['validation_method']
		if self.cord:
			if '--validation-method' not in sys.argv:
				self.validation_method = 'Backbone.Cord.validate'
		if self.es6:
			if '--module-type' not in sys.argv:
				self.module_type = 'esm'
			self.const = 'const'
			self.local = 'let'
		else:
			self.const = 'var'
			self.local = 'var'
		self.indent = options['indent']
		self.emit = CodeEmitter(sys.stdout, self.indent)
		self.regex_stack = []
		self.models = {}
		self.model_properties = {}
		self.model_properties_args = {}
		self.collections = {}
		self.related_collections = {}
		self.singletons = {}
		self.include_related = {}
		self.readonly_fields = {}
		self.readonly_included_fields = {}
		self.camelcase = getattr(settings, 'API_CAMELCASE', True)
		self.renameid = getattr(settings, 'API_RENAME_ID', True)

		module = import_module(settings.ROOT_URLCONF)
		self.enum_patterns(module.urlpatterns)
		# Add newUrl attributes to the models so they can be created outside of collections.
		# Matching models and collections will share the same name
		for name in self.models:
			if self.models[name].has_key('urlRoot') and self.collections.has_key(name) and self.collections[name].has_key('url'):
				self.models[name]['newUrl'] = self.collections[name]['url']

		try:
			os.makedirs(self.dest)
		except:
			print 'Warning: Overwriting any contents in ' + self.dest

		self.dest_models = os.path.join(self.dest, 'models')
		self.dest_collections = os.path.join(self.dest, 'collections')
		try:
			os.mkdir(self.dest_models)
		except:
			pass
		try:
			os.mkdir(self.dest_collections)
		except:
			pass

		# Output each model and collection
		for name in self.models:
			self.output_model(name, self.models[name])
		for name in self.collections:
			self.output_collection(name, self.collections[name])

		eslint_path = os.path.join(os.path.dirname(__file__), '../templates/eslintrc')
		# Copy .eslintrc file
		copyfile(eslint_path, os.path.join(self.dest_models, '.eslintrc'))
		copyfile(eslint_path, os.path.join(self.dest_collections, '.eslintrc'))



from django.contrib.contenttypes.models import ContentType
from django.db.models.query import QuerySet

__METHOD_PERM_DICT = {
	'POST': 'add',
	'PUT': 'change',
	'PATCH': 'change',
	'DELETE': 'delete'
}

def default_permission_authorization(request, obj):
	"""Check default permissions based on the object's model."""
	perm = __METHOD_PERM_DICT.get(request.method, None)
	if perm:
		content_type = ContentType.objects.get_for_model(obj.__class__)
		perm = '%s.%s_%s' % (content_type.app_label, perm, content_type.model)
		return request.user.has_perm(perm)
	return True

def superuser_exempt(filter_authorization_verification):
	"""Decorator to exempt any superuser from filtering, authorization, or verification functions."""
	def superuser_exempt_fun(request, arg):
		if request.user.is_superuser:
			if isinstance(arg, QuerySet):
				return arg
			else:
				return True
		return filter_authorization_verification(request, arg)
	return superuser_exempt_fun



def get_doc_str(fun):
	"""Get the doc string for a function and return a default if none is found."""
	if fun.__doc__:
		return '\n'.join([line.strip() for line in fun.__doc__.split('\n')])
	else:
		return 'No documentation provided for %s()' % fun.__name__

def get_base_classes(c):
	"""Given a class, return all of its base classes as a list."""
	base_classes = list(c.__bases__)
	for b in list(base_classes):
		base_classes.extend(get_base_classes(b))
	return base_classes

def get_base_models(c):
	from django.db import models
	base_classes = get_base_classes(c)
	bases = []
	for cls in base_classes:
		if issubclass(cls, models.Model) and cls is not models.Model and not cls._meta.abstract:
			bases.append(cls)
	return bases

def get_base_model(c):
	bases = get_base_models(c)
	if bases:
		return bases[0]

def get_api_properties(model):
	"""Given a model, gather all of its api properties."""
	properties = {}
	for cls in [model] + list(model._meta.get_parent_list()):
		for field_name in cls.__dict__:
			attr = cls.__dict__[field_name]
			if type(attr) is property and attr.fget and hasattr(attr.fget, 'api_code'):
				properties[field_name] = attr
	return properties

def get_resource_type(regex_stack, pattern):
	"""Given a pattern, determine its type of 'Single Object', 'Object', or 'Collection'."""
	from symmetric.views import BasicApiView
	url = ''.join(regex_stack)
	view = pattern.callback
	single_object = False
	if getattr(view, 'parent_model', None):
		# For api_related_view, ApiRelatedView, or BasicApiView
		if isinstance(view, BasicApiView) and view.single_object:
			return 'Single Object'
		else:
			return 'Collection'
	else:
		# For api_view, ApiView, or BasicApiView (without a parent_model)
		if (url.find('<object_id>') == -1 and url.find('<slug>') == -1):
			if (isinstance(view, BasicApiView) and view.single_object) or (pattern.default_args.has_key('object_id') or pattern.default_args.has_key('slug')):
				return 'Single Object'
			else:
				return 'Collection'
		else:
			return 'Object'

def get_app_url_prefix(app_name, app_patterns):
	"""Get the url prefix for an app's urls."""
	from importlib import import_module
	from django.conf import settings
	module_name = app_name.lower() + '.urls'
	module = import_module(settings.ROOT_URLCONF)
	for urlpattern in module.urlpatterns:
		# A RegexURLResolver object will have a _urlconf_module attribute that is either a module or a RegexURLPattern list matched up with the app_name or app_patterns
		if hasattr(urlpattern, '_urlconf_module'):
			if isinstance(urlpattern._urlconf_module, list):
				if urlpattern._urlconf_module is app_patterns:
					return urlpattern._regex
			elif getattr(urlpattern._urlconf_module, '__name__', '').lower() == module_name:
				return urlpattern._regex
	return ''

def get_field(model, field_name):
	"""Run get_field on the model, but detect _id as a fallback."""
	try:
		return model._meta.get_field(field_name)
	except Exception as e:
		if field_name.endswith('_id'):
			return model._meta.get_field(field_name[:-3])
		else:
			raise e

def has_field(model, field_name, include_inherited=True):
	try:
		field = model._meta.get_field(field_name)
		if include_inherited:
			return True
		else:
			return field.model is model
	except:
		return False

def get_subclass_filter(view):
	from symmetric.filters import subclass_filter, combine_filters
	filter = None
	def _from_combined(combined):
		for temp in combined.filters:
			if isinstance(temp, combine_filters):
				temp = _from_combined(temp)
			if isinstance(temp, subclass_filter):
				return temp
	if hasattr(view, 'filter'):
		if isinstance(view.filter, subclass_filter):
			filter = view.filter
		elif isinstance(view.filter, combine_filters):
			filter = _from_combined(view.filter)
	return filter

def get_model_name(model):
	"""Given a model, return its CamelCase singular name."""
	return model.__name__

def get_model_name_plural(model):
	"""Given a model, return its CamelCase plural name."""
	return verbose_to_camel_case(model._meta.verbose_name_plural)

def get_collection_name(view):
	"""Given a view, return the CamelCase singular name of the collection."""
	filter = get_subclass_filter(view)
	if filter and filter.names and filter.names.has_key('name'):
		return filter.names['name']
	return get_model_name(view.model)

def get_collection_name_plural(view):
	"""Given a view, return the CamelCase plural name of the collection."""
	filter = get_subclass_filter(view)
	if filter and filter.names:
		if filter.names.has_key('name_plural'):
			return filter.names['name_plural']
		elif filter.names.has_key('name'):
			return filter.names['name'] + 's'
	return get_model_name_plural(view.model)

def is_sublist(list, sublist):
	"""Take a list and potential sublist and compare for membership."""
	# l = The length of each comparison
	# n = The number of equal length sublists in list
	l = len(sublist)
	n = len(list) - l + 1
	if n >= 1:
		for i in range(n):
			if sublist == list[i:i+l]:
				return True
	return False

def is_anonymous(view):
	"""Returns True/False depending on if the view allows anonymous access or not."""
	from symmetric.views import ApiRequirement
	if hasattr(view, 'requirements'):
		if view.requirements & ApiRequirement.LOGIN and not view.requirements & ApiRequirement.ANONYMOUS_READ:
			return False
	return True

def is_auto_now(field_name, view):
	"""Return True/False depending on if the field in this view is auto_now."""
	for field in view.model._meta.fields:
		if field.name == field_name:
			if hasattr(field, 'auto_now_add') and field.auto_now_add:
				return True
			if hasattr(field, 'auto_now') and field.auto_now:
				return True
	return False

def is_readonly(model, field_name):
	from symmetric.functions import _ApiModel
	api_model = _ApiModel(model)
	for decoded_name, encoded_name, encode, decode in api_model.fields:
		if field_name == decoded_name:
			return encoded_name not in api_model.encoded_fields
	return True

def is_excluded(model, field_name, flat=True):
	# Skip any ptr field to base models
	if field_name.endswith('_ptr_id'):
		return True
	# Skip any field that is not directly on model and is not the primary id field (which could be on the base too)
	if not flat:
		field = get_field(model, field_name)
		if field.model is not model and not field.primary_key:
			return True
	# Skip anything that is not an api field
	from symmetric.functions import _ApiModel
	api_model = _ApiModel(model)
	found = False
	for decoded_name, encoded_name, encode, decode in api_model.fields:
		if field_name == decoded_name:
			found = True
			break
	return not found

def is_included(model, field_name):
	from django.db.models.fields.related import ForeignKey
	include_related = hasattr(model, 'API') and hasattr(model.API, 'include_related') and field_name in model.API.include_related
	field = get_field(model, field_name)
	return (isinstance(field, ForeignKey) and include_related)

def format_regex_stack(regex_stack):
	"""Format a list or tuple of regex url patterns into a single path."""
	import re
	formatted = ''.join(regex_stack)
	formatted = re.sub('\([^<]*(<[^>]*>).*?\)', '\\1', formatted)
	formatted = formatted.replace('^$','/')
	formatted = formatted.replace('^','/')
	formatted = formatted.replace('?$','/')
	formatted = formatted.replace('$','/')
	formatted = formatted.replace('//','/')
	return formatted

def lower_first(string):
	"""Lower the first character of the string."""
	return string[0].lower() + string[1:]

def verbose_to_camel_case(string):
	"""Covert a Verbose Model Name to CamelCaseModelName."""
	return ''.join([word.capitalize() for word in string.split(' ')])

def camel_case_to_verbose(string):
	"""Convert a CamelCaseModelName to Verbose Model Name."""
	words = []
	start_index = 0
	for index, c in enumerate(string):
		# Ignore the first character
		if c.isupper() and index:
			words.append(string[start_index:index])
			start_index = index
	words.append(string[start_index:])
	return ' '.join(words)



from importlib import import_module
from optparse import make_option
import os

from django.apps import apps
from django.conf import settings
from django.core.management.base import CommandError
from django.db.models.fields import NOT_PROVIDED, TimeField, DateField
from django.db.models.fields.related import ForeignKey
from django.template import Template, Context

import symmetric.management.overrides
from symmetric.functions import _ApiModel, underscore_to_camel_case
from symmetric.management.functions import get_base_classes, get_base_models, get_base_model, get_field, has_field
from symmetric.management.translate import translate_code
from symmetric.models import get_related_model
from symmetric.views import ApiAction, ApiRequirement, BasicApiView, api_view

get_model = apps.get_model

class GenerateModelsCommand(object):
	option_list = (
			make_option('--prefix',
				type='string',
				dest='prefix',
				default='',
				help='Prefix to add to each class name and file name.'),
			make_option('--dest',
				type='string',
				dest='dest',
				help='Output the all detected models from api endpoints and render them into this destination directory.'),
			make_option('--exclude',
				type='string',
				dest='exclude',
				action='append',
				help='Do not output anything for the models specified.')
		)

	def get_include_related_models(self, model):
		related_models = set()
		if hasattr(model, 'API') and hasattr(model.API, 'include_related'):
			include_related = model.API.include_related
			for field in model._meta.fields:
				if field.name in include_related:
					related_models.add(get_related_model(field))
					related_models |= self.get_include_related_models(get_related_model(field))
		return related_models

	def post_render(self, output):
		return output

	def base_extra_context(self, model, api_model):
		has_date = False
		has_bool = False
		datetime_fields = []
		primary_field = None
		if api_model.id_field:
			primary_field = api_model.id_field[1]
		base = get_base_model(model)
		base_name = None
		if base:
			base_name = base.__name__
		for decoded_name, encoded_name, encode, decode in api_model.fields:
			if has_field(base, decoded_name):
				continue
			field = get_field(model, decoded_name)
			field_type = field.__class__.__name__
			if field_type == 'DateTimeField' or field_type == 'DateField':
				has_date = True
				datetime_fields.append((encoded_name, encoded_name[0].upper() + encoded_name[1:]))
			elif field_type == 'BooleanField':
				has_bool = True
			if not primary_field and field.primary_key:
				primary_field = encoded_name
		return {'prefix': self.prefix, 'base_name': base_name, 'name': model.__name__, 'name_lower': model.__name__[0].lower() + model.__name__[1:], 'has_date': has_date, 'has_bool': has_bool, 'primary_field': primary_field, 'datetime_fields': datetime_fields}

	def perform_mapping(self, mapping, format_context):
		if callable(mapping):
			# callable method
			return mapping(format_context)
		elif isinstance(mapping, Template):
			# django template
			return mapping.render(Context(format_context, autoescape=False))
		else:
			# normal python string formatting
			return mapping.format(**format_context)

	def get_context(self, model):
		api_model = _ApiModel(model)
		context = self.base_extra_context(model, api_model)

		if hasattr(self, 'extra_context'):
			context.update(self.extra_context(model, api_model))

		# Loop over the mappings
		for mapping_name in self.mappings:
			mapping = self.mappings[mapping_name]
			write_only = False
			if isinstance(mapping, dict):
				write_only = mapping.get('WriteOnly', False)
			lines = []
			for decoded_name, encoded_name, encode, decode in api_model.fields:
				field = get_field(model, decoded_name)
				# Skip any field that is not directly on model and is not the primary id field (which could be on the base too)
				if field.model is not model and encoded_name != context['primary_field']:
					continue
				# Skip any ptr field to base models
				if decoded_name.endswith('_ptr_id'):
					continue
				include_related = hasattr(model, 'API') and hasattr(model.API, 'include_related') and field.name in model.API.include_related
				included_readonly = False
				included_obj_name = ''
				if write_only and encoded_name not in api_model.encoded_fields:
					# Skip readonly fields, but make an exception for included foreign keys, see Included Objects in the documentation
					if isinstance(field, ForeignKey) and include_related:
						included_obj_name = encoded_name
						encoded_name += 'Id' if self.camelcase else '_id'
						included_readonly = True
					else:
						continue
				line = None
				classes = [field.__class__] + get_base_classes(field.__class__)
				for cls in classes:
					field_type = cls.__name__
					if callable(mapping):
						line = mapping(model, encoded_name, field)
					elif mapping.has_key(field_type):
						format_context = {'name': encoded_name, 'null': field.null}
						if field.default is not NOT_PROVIDED and not isinstance(field, (TimeField, DateField)):
							# Only supply default values for non-date/time fields, it will be easier to just add these after manually
							format_context['default'] = field.default
						if include_related:
							format_context['included'] = True
							format_context['included_readonly'] = included_readonly
							format_context['included_obj_name'] = included_obj_name
							format_context['included_name'] = get_related_model(field).__name__
						line = self.perform_mapping(mapping[field_type], format_context)
					if line is not None:
						break
				if line is None:
					raise CommandError("No such mapping for %s in %s." % (field_type, mapping_name))
				elif line:
					lines += line.split('\n')
			context[mapping_name] = lines

		# Translate the api properties
		if hasattr(self, 'property_declarations') or hasattr(self, 'property_implementations'):
			decl_lines = []
			impl_lines = []
			property_transformer = getattr(self, 'property_transformer', None)
			for name in model.__dict__:
				attr = model.__dict__[name]
				if type(attr) is property and attr.fget and hasattr(attr.fget, 'api_code'):
					if getattr(attr.fget, 'api_translations', None) and attr.fget.api_translations.has_key(self.lang):
						code = attr.fget.api_translations[self.lang]
					else:
						code = translate_code(attr.fget.api_code, self.lang, (property_transformer(model) if property_transformer else None))
					format_context = {'name': name if not self.camelcase else underscore_to_camel_case(name), 'type': self.property_types[attr.fget.api_type], 'code': code}
					format_context['name_upper'] = format_context['name'][0].upper() + format_context['name'][1:]
					if hasattr(self, 'property_declarations'):
						line = self.perform_mapping(self.property_declarations, format_context)
						decl_lines += line.split('\n')
					if hasattr(self, 'property_implementations'):
						line = self.perform_mapping(self.property_implementations, format_context)
						impl_lines += line.split('\n')
			if decl_lines:
				context['property_declarations'] = decl_lines
			if impl_lines:
				context['property_implementations'] = impl_lines

		return context

	def enum_patterns(self, patterns):
		for pattern in patterns:
			if pattern.callback:
				if isinstance(pattern.callback, (api_view, BasicApiView)) and pattern.callback.model:
					self.models.add(pattern.callback.model)
					self.models |= self.get_include_related_models(pattern.callback.model)
			else:
				self.enum_patterns(pattern.url_patterns)

	def expand_mappings(self, field, *expanded_fields):
		for mapping in self.mappings.values():
			for key, value in mapping.items():
				if key == field:
					for expanded_field in expanded_fields:
						if not mapping.has_key(expanded_field):
							mapping[expanded_field] = mapping[field]
					break

	def render(self, *args, **options):
		self.camelcase = getattr(settings, 'API_CAMELCASE', True)
		self.prefix = options['prefix']
		if not hasattr(self, 'templates'):
			raise CommandError('No templates set!')
		if options and options['dest']:
			try:
				os.makedirs(options['dest'])
			except:
				print 'Warning: Overwriting any contents in %s' % options['dest']
			self.models = set()
			module = import_module(settings.ROOT_URLCONF)
			self.enum_patterns(module.urlpatterns)
			# Add any base models to the set
			base_models = set()
			for model in self.models:
				base_models |= set(get_base_models(model))
			self.models |= base_models
			for model in self.models:
				if options['exclude'] and model.__name__ in options['exclude']:
					continue
				context = self.get_context(model)
				for i in range(len(self.templates)):
					template = self.templates[i]
					template_extension = self.template_extensions[i]
					path = os.path.join(options['dest'], '%s%s.%s' % (self.prefix, model.__name__, template_extension))
					print 'Rendering %s' % path
					with open(path, 'w') as f:
						f.write(self.post_render(template.render(Context(context, autoescape=False))))
		elif args:
			for model_name in args:
				model = model_name.split('.')
				model = get_model(model[0], model[1])
				context = self.get_context(model)
				for template in self.templates:
					print self.post_render(template.render(Context(context, autoescape=False)))
		else:
			raise CommandError("No model or destination directory specified.")



import os
from optparse import make_option

from django.core.management.base import BaseCommand
from django.template import Template

from symmetric.management.generateconnections import GenerateConnectionsCommand

WITH_PARAM = '{% if param %}With{{ param }}:({{ param_type }}){{ name_lower }}{{ param }}{% endif %}'
WITH_PARENT_PARAM = '{% if param %}With{{ param }}:({{ param_type }}){{ parent_name_lower }}{{ param }}{% endif %}'
WITH_PARENT_PARAM_PARAMS = '{% if param %}With{{ param }}:({{ param_type }}){{ parent_name_lower }}{{ param }} params:(APIRequestParams *)params{% else %}WithParams:(APIRequestParams *)params{% endif %}'
FOR_PARENT = '{% if not param %}For{{ parent_name }}{% endif %}'
FOR_PARENT_PARAM = '{% if param %} for{{ parent_name }}With{{ param }}:({{ param_type }}){{ parent_name_lower }}{{ param }}{% endif %}'
WITH_PARAM_COMPLETION = '{% if param %}With{{ param }}:({{ param_type }}){{ name_lower }}{{ param }} completionHandler:(APIReadHandler)completion{% else %}WithCompletionHandler:(APIReadHandler)completion{% endif %}'
WITH_PARENT_PARAM_COMPLETION = '{% if param %}With{{ param }}:({{ param_type }}){{ parent_name_lower }}{{ param }} completionHandler:(APIReadHandler)completion{% else %}WithCompletionHandler:(APIReadHandler)completion{% endif %}'

READ_OBJECT_METHOD = """
- (void)read{{ name }}/1
{
	NSString *path = {% if param %}[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ name|upper }}_WITH_{{ param|upper }}, {{ name_lower }}{{ param }}]{% else %}URL_{{ app_name|upper }}_{{ name|upper }}{% endif %};
	[self performRequestWithObject:nil action:API_ACTION_READ requestType:REQUEST_{{ app_name|upper }}_READ_{{ name|upper }} path:path https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:NO];
}

- (void)read{{ name }}/2
{
	self.readHandler = completion;
	[self read{{ name }}{% if param %}With{{ param }}:{{ name_lower }}{{ param }}{% endif %}];
}""".replace('/1', WITH_PARAM).replace('/2', WITH_PARAM_COMPLETION)

READ_LIST_METHOD = """
- (void)read{{ name_plural }}
{
	[self performRequestWithObject:nil action:API_ACTION_LIST requestType:REQUEST_{{ app_name|upper }}_READ_{{ name_plural|upper }} path:URL_{{ app_name|upper }}_{{ name_plural|upper }} https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:NO];
}

- (void)read{{ name_plural }}WithParams:(APIRequestParams *)params
{
	self.requestParams = params;
	[self read{{ name_plural }}];
}

- (void)read{{ name_plural }}WithParams:(APIRequestParams *)params completionHandler:(APIListHandler)completion
{
	self.requestParams = params;
	self.listHandler = completion;
	[self read{{ name_plural }}];
}"""

CREATE_METHOD = """
- (void)create{{ name }}:({{ name }} *){{ name_lower }}
{
	[self performRequestWithObject:{{ name_lower }} action:API_ACTION_CREATE requestType:REQUEST_{{ app_name|upper }}_CREATE_{{ name|upper }} path:URL_{{ app_name|upper }}_{{ name_plural|upper }} https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:{% if hmac %}YES{% else %}NO{% endif %}];
}

- (void)create{{ name }}:({{ name }} *){{ name_lower }} completionHandler:(APIHandler)completion
{
	self.completionHandler = completion;
	[self create{{ name }}:{{ name_lower }}];
}"""

UPDATE_METHOD = """
- (void)update{{ name }}:({{ name }} *){{ name_lower }}
{
	NSString *path = {% if param %}[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ name|upper }}, [{{ name_lower }} objectId]]{% else %}URL_{{ app_name|upper }}_{{ name|upper }}{% endif %};
	[self performRequestWithObject:{{ name_lower }} action:API_ACTION_UPDATE requestType:REQUEST_{{ app_name|upper }}_UPDATE_{{ name|upper }} path:path https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:{% if hmac %}YES{% else %}NO{% endif %}];
}

- (void)update{{ name }}:({{ name }} *){{ name_lower }} completionHandler:(APIHandler)completion
{
	self.completionHandler = completion;
	[self update{{ name }}:{{ name_lower }}];
}"""

DELETE_METHOD = """
- (void)delete{{ name }}:({{ name }} *){{ name_lower }}
{
	[self performRequestWithObject:nil action:API_ACTION_DELETE requestType:REQUEST_{{ app_name|upper }}_DELETE_{{ name|upper }} path:[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ name|upper }}, [{{ name_lower }} objectId]] https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:NO];
}

- (void)delete{{ name }}:({{ name }} *){{ name_lower }} completionHandler:(APIHandler)completion
{
	self.completionHandler = completion;
	[self delete{{ name }}:{{ name_lower }}];
}"""

READ_RELATED_LIST_METHOD = """
- (void)read{{ name_plural }}For{{ parent_name }}/1
{
	NSString *path = {% if param %}[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name_plural|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}]{% else %}URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name_plural|upper }}{% endif %};
	[self performRequestWithObject:nil action:API_ACTION_LIST requestType:REQUEST_{{ app_name|upper }}_READ_{{ parent_name|upper }}_{{ name_plural|upper }} path:path https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:NO];
}

- (void)read{{ name_plural }}For{{ parent_name }}/2
{
	self.requestParams = params;
	[self read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}:{{ parent_name_lower }}{{ param }}{% endif %}];
}

- (void)read{{ name_plural }}For{{ parent_name }}/2 completionHandler:(APIListHandler)completion
{
	self.requestParams = params;
	self.listHandler = completion;
	[self read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}:{{ parent_name_lower }}{{ param }}{% endif %}];
}""".replace('/1', WITH_PARENT_PARAM).replace('/2', WITH_PARENT_PARAM_PARAMS)

CREATE_RELATED_METHOD = """
- (void)create{{ name }}/1:({{ name }} *){{ name_lower }}/2
{
	NSString *path = {% if param %}[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name_plural|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}]{% else %}URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name_plural|upper }}{% endif %};
	[self performRequestWithObject:{{ name_lower }} action:API_ACTION_CREATE requestType:REQUEST_{{ app_name|upper }}_CREATE_{{ parent_name|upper }}_{{ name|upper }} path:path https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:{% if hmac %}YES{% else %}NO{% endif %}];
}

- (void)create{{ name }}/1:({{ name }} *){{ name_lower }}/2 completionHandler:(APIHandler)completion
{
	self.completionHandler = completion;
	{% if param %}[self create{{ name }}:{{ name_lower }} for{{ parent_name }}With{{ param }}:{{ parent_name_lower }}{{ param }}]{% else %}[self create{{ name }}For{{ parent_name }}:{{ name_lower }}]{% endif %};
}""".replace('/1', FOR_PARENT).replace('/2', FOR_PARENT_PARAM)

READ_RELATED_OBJECT_METHOD = """
- (void)read{{ name }}For{{ parent_name }}/1
{
	NSString *path = {% if param %}[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}]{% else %}URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name|upper }}{% endif %};
	[self performRequestWithObject:nil action:API_ACTION_READ requestType:REQUEST_{{ app_name|upper }}_READ_{{ parent_name|upper }}_{{ name|upper }} path:path https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:NO];
}

- (void)read{{ name }}For{{ parent_name }}/2
{
	self.readHandler = completion;
	[self read{{ name }}For{{ parent_name }}{% if param %}With{{ param }}:{{ parent_name_lower }}{{ param }}{% endif %}];
}""".replace('/1', WITH_PARENT_PARAM).replace('/2', WITH_PARENT_PARAM_COMPLETION)

UPDATE_RELATED_METHOD = """
- (void)update{{ name }}/1:({{ name }} *){{ name_lower }}/2
{
	NSString *path = {% if param %}[NSString stringWithFormat:URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}]{% else %}URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name|upper }}{% endif %};
	[self performRequestWithObject:{{ name_lower }} action:API_ACTION_UPDATE requestType:REQUEST_{{ app_name|upper }}_UPDATE_{{ parent_name|upper }}_{{ name|upper }} path:path https:{% if https %}YES{% else %}NO{% endif %} login:{% if login %}YES{% else %}NO{% endif %} sign:{% if hmac %}YES{% else %}NO{% endif %}];
}

- (void)update{{ name }}/1:({{ name }} *){{ name_lower }}/2 completionHandler:(APIHandler)completion
{
	self.completionHandler = completion;
	[self update{{ name }}{% if param %} for{{ parent_name }}With{{ param }}:{{ parent_name_lower }}{{ param }}{% else %}For{{ parent_name }}:{{ name_lower }}{% endif %}];
}""".replace('/1', FOR_PARENT).replace('/2', FOR_PARENT_PARAM)

READ_OBJECT_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_READ_{{ name|upper }}:
				responseObject = [self processResponseWithClass:[{{ name }} class]];
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didRead{{ name }}:) object:responseObject];
				[responseObject release];
				break;"""

READ_LIST_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_READ_{{ name_plural|upper }}:
				{% if subclasses %}responseArray = [self processResponseWithClasses:@{ {% for subclass in subclasses %}@"{{ subclass.0 }}" : [@"{{ subclass.1 }}" class]{% endfor %} }];{% else %}responseArray = [self processResponseWithClass:[{{ name }} class]];{% endif %}
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didRead{{ name_plural }}:) object:responseArray];
				[responseArray release];
				break;"""

CREATE_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_CREATE_{{ name|upper }}:
				[tempRequestObject setObjectId:_newObjectId];
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didCreate{{ name }}:) object:tempRequestObject];
				break;"""

UPDATE_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_UPDATE_{{ name|upper }}:
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didUpdate{{ name }}:) object:tempRequestObject];
				break;"""

DELETE_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_DELETE_{{ name|upper }}:
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didDelete{{ name }}:) object:tempRequestObject];
				break;"""

READ_RELATED_LIST_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_READ_{{ parent_name|upper }}_{{ name_plural|upper }}:
				{% if subclasses %}responseArray = [self processResponseWithClasses:@{ {% for subclass in subclasses %}@"{{ subclass.0 }}" : [@"{{ subclass.1 }}" class]{% endfor %} }];{% else %}responseArray = [self processResponseWithClass:[{{ name }} class]];{% endif %}
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didRead{{ name_plural }}For{{ parent_name }}:) object:responseArray];
				[responseArray release];
				break;"""

CREATE_RELATED_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_CREATE_{{ parent_name|upper }}_{{ name|upper }}:
				[tempRequestObject setObjectId:_newObjectId];
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didCreate{{ name }}For{{ parent_name }}:) object:tempRequestObject];
				break;"""

READ_RELATED_OBJECT_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_READ_{{ parent_name|upper }}_{{ name|upper }}:
				responseObject = [self processResponseWithClass:[{{ name }} class]];
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didRead{{ name }}For{{ parent_name }}:) object:responseObject];
				[responseObject release];
				break;"""

UPDATE_RELATED_RESPONSE = """
			case REQUEST_{{ app_name|upper }}_UPDATE_{{ parent_name|upper }}_{{ name|upper }}:
				self.responseData = nil;
				[self completeWithSelector:@selector({{ app_name_lower }}Connection:didUpdate{{ name }}For{{ parent_name }}:) object:tempRequestObject];
				break;"""

class Command(BaseCommand, GenerateConnectionsCommand):
	help = 'Generate iOS connections for API endpoints.'
	option_list = BaseCommand.option_list + GenerateConnectionsCommand.option_list + (
			make_option('--connection',
				type='string',
				dest='connection',
				default='APIURLConnection',
				help='Connection class to use that should have the same interface as NSURLConnection.'),
		)

	def extra_context(self):
		return {'connection': self.connection }

	def handle(self, *args, **options):
		# Remember options
		self.connection = options['connection']

		# Create the templates
		self.templates = []
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'ios-connection.h')
		with open(template_path) as f:
			self.templates.append(Template(f.read()))
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'ios-connection.m')
		with open(template_path) as f:
			self.templates.append(Template(f.read()))
		self.template_extensions = ['h', 'm']

		# Create the mappings
		import_template = Template('#import "{{ name }}.h"')
		imports_mapping = dict.fromkeys(GenerateConnectionsCommand.ALL_ACTIONS, import_template)
		imports_mapping['RemoveDuplicates'] = True
		imports_mapping['Sort'] = True

		request_types_base = 'REQUEST_{{ app_name|upper }}_%s_%s'
		request_types_mapping = {
			'READ_OBJECT': Template(request_types_base % ('READ', '{{ name|upper }}')),
			'READ_LIST': Template(request_types_base % ('READ', '{{ name_plural|upper }}')),
			'CREATE': Template(request_types_base % ('CREATE', '{{ name|upper }}')),
			'UPDATE': Template(request_types_base % ('UPDATE', '{{ name|upper }}')),
			'DELETE': Template(request_types_base % ('DELETE', '{{ name|upper }}')),
			'READ_RELATED_LIST': Template(request_types_base % ('READ', '{{ parent_name|upper }}_{{ name_plural|upper }}')),
			'CREATE_RELATED': Template(request_types_base % ('CREATE', '{{ parent_name|upper }}_{{ name|upper }}')),
			'READ_RELATED_OBJECT': Template(request_types_base % ('READ', '{{ parent_name|upper }}_{{ name|upper }}')),
			'UPDATE_RELATED': Template(request_types_base % ('UPDATE', '{{ parent_name|upper }}_{{ name|upper }}')),
			'RemoveDuplicates': True,
			'Sort': True
		}

		delegate_method_decls_base = "- (void){{ app_name_lower }}Connection:({{ app_name }}Connection *)connection did%s;"
		delegate_method_decls_mapping = {
			'READ_OBJECT': Template(delegate_method_decls_base % ('Read{{ name }}:({{ name }} *){{ name_lower }}')),
			'READ_LIST': Template(delegate_method_decls_base % ('Read{{ name_plural }}:(NSArray *){{ name_plural_lower }}')),
			'CREATE': Template(delegate_method_decls_base % 'Create{{ name }}:({{ name }} *){{ name_lower }}'),
			'UPDATE': Template(delegate_method_decls_base % 'Update{{ name }}:({{ name }} *){{ name_lower }}'),
			'DELETE': Template(delegate_method_decls_base % 'Delete{{ name }}:({{ name }} *){{ name_lower }}'),
			'READ_RELATED_LIST': Template(delegate_method_decls_base % ('Read{{ name_plural }}For{{ parent_name }}:(NSArray *){{ name_plural_lower }}')),
			'CREATE_RELATED': Template(delegate_method_decls_base % ('Create{{ name }}For{{ parent_name }}:({{ name }} *){{ name_lower }}')),
			'READ_RELATED_OBJECT': Template(delegate_method_decls_base % ('Read{{ name }}For{{ parent_name }}:({{ name }} *){{ name_lower }}')),
			'UPDATE_RELATED': Template(delegate_method_decls_base % ('Update{{ name }}For{{ parent_name }}:({{ name }} *){{ name_lower }}')),
			'RemoveDuplicates': True,
			'Sort': True
		}

		method_decls_mapping = {
			'READ_OBJECT': Template('- (void)read{{ name }}%s;' % WITH_PARAM),
			'READ_LIST': Template('- (void)read{{ name_plural }};\n- (void)read{{ name_plural }}WithParams:(APIRequestParams *)params;'),
			'CREATE': Template('- (void)create{{ name }}:({{ name }} *){{ name_lower }};'),
			'UPDATE': Template('- (void)update{{ name }}:({{ name }} *){{ name_lower }};'),
			'DELETE': Template('- (void)delete{{ name }}:({{ name }} *){{ name_lower }};'),
			'READ_RELATED_LIST': Template('- (void)read{{ name_plural }}For{{ parent_name }}%s;\n- (void)read{{ name_plural }}For{{ parent_name }}%s;' % (WITH_PARENT_PARAM, WITH_PARENT_PARAM_PARAMS)),
			'CREATE_RELATED': Template('- (void)create{{ name }}%s:({{ name }} *){{ name_lower }}%s;' % (FOR_PARENT, FOR_PARENT_PARAM)),
			'READ_RELATED_OBJECT': Template('- (void)read{{ name }}For{{ parent_name }}%s;' % WITH_PARENT_PARAM),
			'UPDATE_RELATED': Template('- (void)update{{ name }}%s:({{ name }} *){{ name_lower }}%s;' % (FOR_PARENT, FOR_PARENT_PARAM)),
			'Sort': True
		}

		block_method_decls_mapping = {
			'READ_OBJECT': Template('- (void)read{{ name }}%s;' % WITH_PARAM_COMPLETION),
			'READ_LIST': Template('- (void)read{{ name_plural }}WithParams:(APIRequestParams *)params completionHandler:(APIListHandler)completion;'),
			'CREATE': Template('- (void)create{{ name }}:({{ name }} *){{ name_lower }} completionHandler:(APIHandler)completion;'),
			'UPDATE': Template('- (void)update{{ name }}:({{ name }} *){{ name_lower }} completionHandler:(APIHandler)completion;'),
			'DELETE': Template('- (void)delete{{ name }}:({{ name }} *){{ name_lower }} completionHandler:(APIHandler)completion;'),
			'READ_RELATED_LIST': Template('- (void)read{{ name_plural }}For{{ parent_name }}%s completionHandler:(APIListHandler)completion;' % WITH_PARENT_PARAM_PARAMS),
			'CREATE_RELATED': Template('- (void)create{{ name }}%s:({{ name }} *){{ name_lower }}%s completionHandler:(APIHandler)completion;' % (FOR_PARENT, FOR_PARENT_PARAM)),
			'READ_RELATED_OBJECT': Template('- (void)read{{ name }}For{{ parent_name }}%s;' % WITH_PARENT_PARAM_COMPLETION),
			'UPDATE_RELATED': Template('- (void)update{{ name }}%s:({{ name }} *){{ name_lower }}%s completionHandler:(APIHandler)completion;' % (FOR_PARENT, FOR_PARENT_PARAM)),
			'Sort': True
		}

		urls_mapping = {}
		urls_mapping['READ_OBJECT'] = Template('#define URL_{{ app_name|upper }}_{{ name|upper }}{% if param %}_WITH_{{ param|upper }}{% endif %} @"{{ url_format }}"')
		urls_mapping['READ_LIST'] = urls_mapping['CREATE'] = Template('#define URL_{{ app_name|upper }}_{{ name_plural|upper }} @"{{ url_format }}"')
		urls_mapping['UPDATE'] = urls_mapping['DELETE'] = Template('#define URL_{{ app_name|upper }}_{{ name|upper }} @"{{ url_format }}"')
		urls_mapping['READ_RELATED_LIST'] = urls_mapping['CREATE_RELATED'] = Template('#define URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name_plural|upper }}{% if param %}_WITH_{{ param|upper }}{% endif %} @"{{ url_format }}"')
		urls_mapping['READ_RELATED_OBJECT'] = urls_mapping['UPDATE_RELATED'] = Template('#define URL_{{ app_name|upper }}_{{ parent_name|upper }}_{{ name|upper }}{% if param %}_WITH_{{ param|upper }}{% endif %} @"{{ url_format }}"')
		urls_mapping['RemoveDuplicates'] = True

		methods_mapping = {
			'READ_OBJECT': Template(READ_OBJECT_METHOD),
			'READ_LIST': Template(READ_LIST_METHOD),
			'CREATE': Template(CREATE_METHOD),
			'UPDATE': Template(UPDATE_METHOD),
			'DELETE': Template(DELETE_METHOD),
			'READ_RELATED_LIST': Template(READ_RELATED_LIST_METHOD),
			'CREATE_RELATED': Template(CREATE_RELATED_METHOD),
			'READ_RELATED_OBJECT': Template(READ_RELATED_OBJECT_METHOD),
			'UPDATE_RELATED': Template(UPDATE_RELATED_METHOD)
		}

		response_cases_mapping = {
			'READ_OBJECT': Template(READ_OBJECT_RESPONSE),
			'READ_LIST': Template(READ_LIST_RESPONSE),
			'CREATE': Template(CREATE_RESPONSE),
			'UPDATE': Template(UPDATE_RESPONSE),
			'DELETE': Template(DELETE_RESPONSE),
			'READ_RELATED_LIST': Template(READ_RELATED_LIST_RESPONSE),
			'CREATE_RELATED': Template(CREATE_RELATED_RESPONSE),
			'READ_RELATED_OBJECT': Template(READ_RELATED_OBJECT_RESPONSE),
			'UPDATE_RELATED': Template(UPDATE_RELATED_RESPONSE),
			'RemoveDuplicates': True
		}

		# Set the configuration options and render
		self.mappings = {'imports': imports_mapping, 'request_types': request_types_mapping, 'delegate_method_decls': delegate_method_decls_mapping, 'method_decls': method_decls_mapping, 'block_method_decls': block_method_decls_mapping, 'urls': urls_mapping, 'methods': methods_mapping, 'response_cases': response_cases_mapping }
		self.object_id_format = "%u"
		self.slug_format = "%@"
		self.object_id_type = "uint32_t"
		self.slug_type = "NSString *"
		self.render(*args, **options)



from django.apps import apps

import symmetric.filters
import symmetric.views
from symmetric.views import ApiAction, ApiRequirement

get_model = apps.get_model

"""
Override most of the api views to collect information about project's API.
This file must be imported before accessing any of the views getting overridden!
"""

class _api_view(object):
	def __init__(self, model, actions=ApiAction.READ, requirements=0, filter=None, authorization=None, verification=None):
		if isinstance(model, (str, unicode)):
			model = model.split('.')
			self.model = get_model(model[0], model[1])
		else:
			self.model = model
		self.actions = actions
		self.requirements = requirements
		self.filter = filter
		self.authorization = authorization
		self.verification = verification

	def __call__(self):
		# Empty call so that django will accept is as a view
		pass

class _subclass_filter(object):
	def __init__(self, *subclasses, **names):
		subclasses = list(subclasses)
		for i, subclass in enumerate(subclasses):
			if isinstance(subclass, (str, unicode)):
				subclass = subclass.split('.')
				subclasses[i] = get_model(subclass[0], subclass[1])
		self.subclasses = subclasses
		self.names = names

	def __str__(self):
		subclass_names = [cls.__name__ for cls in self.subclasses]
		return 'Generate a mixed result set of ' + ', '.join(subclass_names) + ' objects'

class _combine_filters(object):
	def __init__(self, *filters):
		self.filters = filters

	def __str__(self):
		info = []
		for filter in self.filters:
			if isinstance(filter, (_combine_filters, _subclass_filter)):
				info.extend(str(filter).split('\n'))
			else:
				info.append(get_doc_str(filter))
		return '\n'.join(info)

class _api_related_view(_api_view):
	def __init__(self, model, related_model, related_field, actions=ApiAction.READ, requirements=0, filter=None, authorization=None, verification=None):
		if isinstance(model, (str, unicode)):
			model = model.split('.')
			self.parent_model = get_model(model[0], model[1])
		else:
			self.parent_model = model
		self.related_field = related_field
		def related_view_filter():
			"""Only return objects associated with the parent model."""
			pass
		if filter:
			filter = _combine_filters(related_view_filter, filter)
		else:
			filter = related_view_filter
		super(_api_related_view, self).__init__(related_model, actions, requirements, filter, authorization, verification)

	def __call__(self):
		# Empty call so that django will accept is as a view
		pass

class _api_filter_contacts_view(object):
	def __init__(self, requirements=ApiRequirement.LOGIN|ApiRequirement.HTTPS, fields=('email',)):
		self.requirements = requirements
		self.fields = fields
		self.__doc__ = _api_filter_contacts_view.doc

	def __call__(self):
		# Empty call so that django will accept is as a view
		pass

symmetric.views.api_view = _api_view
symmetric.views.api_related_view = _api_related_view
_api_filter_contacts_view.doc = symmetric.views.api_filter_contacts_view.__doc__
symmetric.views.api_filter_contacts_view = _api_filter_contacts_view
symmetric.filters.subclass_filter = _subclass_filter
symmetric.filters.combine_filters = _combine_filters



from importlib import import_module

from django.core.urlresolvers import reverse
from django.conf.urls import include, url

def api_include(urlconf_module, namespace=None, app_name=None):
	"""Include api patterns from a module.

	Similar to include() but extracts a variable called apipatterns from urlconf_module and passes that to include().
	The first argument must be a module name that has apipatterns.
	If apipatterns isn't defined, the module itself will be passed to include().
	"""
	if isinstance(urlconf_module, (str, unicode)):
		urlconf_module = import_module(urlconf_module)
	return include(getattr(urlconf_module, 'apipatterns', urlconf_module), namespace, app_name)

def api_patterns(*args):
	"""Create and add api patterns to urlpatterns.

	Args are a module string or a tuple e.g.
	urlpatterns += api_patterns('game.urls', 'cart.urls')
	or api_patterns('game.urls', ('cart', 'cart.urls'), ('versioned', 'versioned.urls', 3))
	or api_patterns(('versioned', 'versioned.urls', 3))
	The latter examples will create the following versioned urls: /api/versioned/, /api/1/versioned/, ..2, and ..3
	"""
	apipatterns = []
	for pattern in args:
		latest_version = 0
		if isinstance(pattern, (str, unicode)):
			api_path = ''
			url_module = pattern
		elif isinstance(pattern, tuple):
			api_path = pattern[0]
			if api_path:
				api_path += '/'
			url_module = pattern[1]
			if len(pattern) >= 3:
				latest_version = pattern[2]
		else:
			continue
		included = api_include(url_module)
		apipatterns.append(url('^api/' + api_path, included))
		if latest_version:
			for i in range(1, latest_version + 1):
				apipatterns.append(url('^api/%d/%s' % (i, api_path), included))
	return apipatterns

def __api_reverse_suffix(path):
	"""Return the normalized suffix of a url without any api information so that the correct version can be added."""
	if path.startswith('/api/'):
		components = path.split('/', 3)
		if len(components) >= 4 and components[2].isdigit():
			return '/' + components[3]
		else:
			return path[4:]
	else:
		return path

def api_reverse(viewname, version=0, urlconf=None, args=None, kwargs=None, prefix=None, current_app=None):
	path = reverse(viewname, args=args, kwargs=kwargs)
	path = __api_reverse_suffix(path)
	if version > 0:
		return ('/api/%d' % version) + path
	else:
		return '/api' + path



import os
from optparse import make_option

from django.core.management.base import BaseCommand
from django.template import Template

from symmetric.management.generatemodels import GenerateModelsCommand

class Command(BaseCommand, GenerateModelsCommand):
	help = 'Generate Windows Phone models for API models or endpoints.'
	option_list = BaseCommand.option_list + GenerateModelsCommand.option_list + (
			make_option('--namespace',
				action='store',
				dest='namespace',
				type='string',
				default='com.business.app',
				help='Set the Windows namespace.'),
		)

	def extra_context(self, model, api_model):
		return {'namespace': self.namespace }

	def handle(self, *args, **options):
		# Remember options
		self.namespace = options['namespace']

		# Create the templates
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'windows-model.cs')
		with open(template_path) as f:
			self.templates = [Template(f.read())]
		self.template_extensions = ['cs']

		# Create the mappings
		property_base = '[DataMember(Name = "{{ name }}")]\ninternal %s {{ name|title }} {%% templatetag openbrace %%} get; set; {%% templatetag closebrace %%}'
		property_mapping = {
			'CharField': Template(property_base % 'string'),
			'IntegerField': Template(property_base % 'int'),
			'PositiveIntegerField': Template(property_base % 'uint'),
			'BigIntegerField': Template(property_base % 'long'),
			'DateField': Template(property_base % 'DateTime'),
			'DecimalField': Template(property_base % 'float'),
			'BooleanField': Template(property_base % 'bool'),
			'Field': ';'
		}
		self.mappings = {'properties': property_mapping }
		self.render(*args, **options)



import os
from optparse import make_option

from django.core.management.base import BaseCommand
from django.template import Template

from symmetric.management.generateconnections import GenerateConnectionsCommand

READ_OBJECT_METHOD = """
	public void read{{ name }}{% if param %}With{{ param }}({{ param_type }} {{ name_lower }}{{ param }}){% else %}(){% endif %}
	{
		String path = {% if param %}String.format(URL_{{ name|upper }}_WITH_{{ param|upper }}, {{ name_lower }}{{ param }}){% else %}URL_{{ name|upper }}{% endif %};
		performRequestWithObject(null, API.ACTION_READ, REQUEST_READ_{{ name|upper }}, path, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, false);
	}

	public static void read{{ name }}{% if param %}With{{ param }}({{ param_type }} {{ name_lower }}{{ param }}, {% else %}({% endif %}RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.read{{ name }}{% if param %}With{{ param }}({{ name_lower }}{{ param }}){% else %}(){% endif %};
	}"""

READ_LIST_METHOD = """
	public void read{{ name_plural }}()
	{
		performRequestWithObject(null, API.ACTION_LIST, REQUEST_READ_{{ name_plural|upper }}, URL_{{ name_plural|upper }}, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, false);
	}

	public void read{{ name_plural }}(APIRequestParams params)
	{
		this.requestParams = params;
		read{{ name_plural }}();
	}

	public static void read{{ name_plural }}(APIRequestParams params, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.requestParams = params;
		connection.read{{ name_plural }}();
	}"""

CREATE_METHOD = """
	public void create{{ name }}({{ name }} {{ name_lower }})
	{
		performRequestWithObject({{ name_lower }}, API.ACTION_CREATE, REQUEST_CREATE_{{ name|upper }}, URL_{{ name_plural|upper }}, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, {% if hmac %}true{% else %}false{% endif %});
	}

	public static void create{{ name }}({{ name }} {{ name_lower }}, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.create{{ name }}({{ name_lower }});
	}"""

UPDATE_METHOD = """
	public void update{{ name }}({{ name }} {{ name_lower }})
	{
		String path = {% if param %}String.format(URL_{{ name|upper }}, {{ name_lower }}.getObjectId()){% else %}URL_{{ name|upper }}{% endif %};
		performRequestWithObject({{ name_lower }}, API.ACTION_UPDATE, REQUEST_UPDATE_{{ name|upper }}, path, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, {% if hmac %}true{% else %}false{% endif %});
	}

	public static void update{{ name }}({{ name }} {{ name_lower }}, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.update{{ name }}({{ name_lower }});
	}"""

DELETE_METHOD = """
	public void delete{{ name }}({{ name }} {{ name_lower }})
	{
		performRequestWithObject(null, API.ACTION_DELETE, REQUEST_DELETE_{{ name|upper }}, String.format(URL_{{ name|upper }}, {{ name_lower }}.getObjectId()), {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, false);
	}

	public static void delete{{ name }}({{ name }} {{ name_lower }}, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.delete{{ name }}({{ name_lower }});
	}"""

READ_RELATED_LIST_METHOD = """
	public void read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}({{ param_type }} {{ parent_name_lower }}{{ param }}){% else %}(){% endif %}
	{
		String path = {% if param %}String.format(URL_{{ parent_name|upper }}_{{ name_plural|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}){% else %}URL_{{ parent_name|upper }}_{{ name_plural|upper }}{% endif %};
		performRequestWithObject(null, API.ACTION_LIST, REQUEST_READ_{{ parent_name|upper }}_{{ name_plural|upper }}, path, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, false);
	}

	public void read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}({{ param_type }} {{ parent_name_lower }}{{ param }}, {% else %}({% endif %}APIRequestParams params)
	{
		this.requestParams = params;
		read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}({{ parent_name_lower }}{{ param }}){% else %}(){% endif %};
	}

	public static void read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}({{ param_type }} {{ parent_name_lower }}{{ param }}, {% else %}({% endif %}APIRequestParams params, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.requestParams = params;
		connection.read{{ name_plural }}For{{ parent_name }}{% if param %}With{{ param }}({{ parent_name_lower }}{{ param }}){% else %}(){% endif %};
	}"""

CREATE_RELATED_METHOD = """
	public void create{{ name }}For{{ parent_name }}{% if param %}With{{ param }}{% endif %}({{ name }} {{ name_lower }}{% if param %}, {{ param_type }} {{ parent_name_lower }}{{ param }}{% endif %})
	{
		String path = {% if param %}String.format(URL_{{ parent_name|upper }}_{{ name_plural|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}){% else %}URL_{{ parent_name|upper }}_{{ name_plural|upper }}{% endif %};
		performRequestWithObject({{ name_lower }}, API.ACTION_CREATE, REQUEST_CREATE_{{ parent_name|upper }}_{{ name|upper }}, path, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, {% if hmac %}true{% else %}false{% endif %});
	}

	public static void create{{ name }}For{{ parent_name }}{% if param %}With{{ param }}{% endif %}({{ name }} {{ name_lower }}{% if param %}, {{ param_type }} {{ parent_name_lower }}{{ param }}{% endif %}, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.create{{ name }}For{{ parent_name }}{% if param %}With{{ param }}({{ name_lower }}, {{ parent_name_lower }}{{ param }}){% else %}({{ name_lower }}){% endif %};
	}"""

READ_RELATED_OBJECT_METHOD = """
	public void read{{ name }}For{{ parent_name }}{% if param %}With{{ param }}({{ param_type }} {{ parent_name_lower }}{{ param }}){% else %}(){% endif %}
	{
		String path = {% if param %}String.format(URL_{{ parent_name|upper }}_{{ name|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}){% else %}URL_{{ parent_name|upper }}_{{ name|upper }}{% endif %};
		performRequestWithObject(null, API.ACTION_READ, REQUEST_READ_{{ parent_name|upper }}_{{ name|upper }}, path, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, false);
	}

	public void read{{ name }}For{{ parent_name }}{% if param %}With{{ param }}({{ param_type }} {{ parent_name_lower }}{{ param }}, RequestListener listener){% else %}(RequestListener listener){% endif %}
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.read{{ name }}For{{ parent_name }}{% if param %}With{{ param }}({{ parent_name_lower }}{{ param }}){% else %}(){% endif %});
	}"""

UPDATE_RELATED_METHOD = """
	public void update{{ name }}For{{ parent_name }}{% if param %}With{{ param }}{% endif %}({{ name }} {{ name_lower }}{% if param %}, ({{ param_type }} {{ parent_name_lower }}{{ param }}{% endif %})
	{
		String path = {% if param %}String.format(URL_{{ parent_name|upper }}_{{ name|upper }}_WITH_{{ param|upper }}, {{ parent_name_lower }}{{ param }}){% else %}URL_{{ parent_name|upper }}_{{ name|upper }}{% endif %};
		performRequestWithObject({{ name_lower }}, API.ACTION_UPDATE, REQUEST_UPDATE_{{ parent_name|upper }}_{{ name|upper }}, path, {% if https %}true{% else %}false{% endif %}, {% if login %}true{% else %}false{% endif %}, {% if hmac %}true{% else %}false{% endif %});
	}

	public static void update{{ name }}For{{ parent_name }}{% if param %}With{{ param }}{% endif %}({{ name }} {{ name_lower }}{% if param %}, ({{ param_type }} {{ parent_name_lower }}{{ param }}{% endif %}, RequestListener listener)
	{
		{{ app_name }}Connection connection = new {{ app_name }}Connection(listener);
		connection.update{{ name }}For{{ parent_name }}{% if param %}With{{ param }}({{ name_lower }}, {{ parent_name_lower }}{{ param }}){% else %}({{ name_lower }}){% endif %};
	}"""

def indent(s, indentation):
	return '\n'.join([indentation + line for line in s.splitlines()])

READ_OBJECT_RESPONSE = indent("""
case REQUEST_READ_{{ name|upper }}:
	responseObject = new {{ name }}(new JSONObject(response));
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidRead{{ name }}({{ app_name }}Connection.this, ({{ name }})responseObject); } });
	break;""", '\t' * 7)

READ_LIST_RESPONSE = indent("""
case REQUEST_READ_{{ name_plural|upper }}:
	jsonArray = new JSONArray(response);
	responseArray = new {{ name }}[jsonArray.length()];
	for(int i = 0; i < jsonArray.length(); ++i) {% templatetag openbrace %}{% if subclasses %}
		JSONObject jsonObject = jsonArray.getJSONObject(i);{% for subclass in subclasses %}
		{% if forloop.counter0 %}else {% endif %}if(jsonObject.has("{{ subclass.0 }}"))
			responseArray[i] = new {{ subclass.1 }}(jsonObject);{% endfor %}{% else %}
		responseArray[i] = new {{ name }}(jsonArray.getJSONObject(i));{% endif %}
	}
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidRead{{ name_plural }}({{ app_name }}Connection.this, ({{ name }}[])responseArray); } });
	break;""", '\t' * 7)

CREATE_RESPONSE = indent("""
case REQUEST_CREATE_{{ name|upper }}:
	(({{ name }}){{ app_name }}Connection.this.requestObject).setObjectId(API.parseInt({{ app_name }}Connection.this.connection.getHeaderField(XHEADER_NEW_OBJECT_ID)));
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidCreate{{ name }}({{ app_name }}Connection.this, ({{ name }}){{ app_name }}Connection.this.requestObject); } });
	break;""", '\t' * 7)

UPDATE_RESPONSE = indent("""
case REQUEST_UPDATE_{{ name|upper }}:
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidUpdate{{ name }}({{ app_name }}Connection.this, ({{ name }}){{ app_name }}Connection.this.requestObject); } });
	break;""", '\t' * 7)

DELETE_RESPONSE = indent("""
case REQUEST_DELETE_{{ name|upper }}:
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidDelete{{ name }}({{ app_name }}Connection.this, ({{ name }}){{ app_name }}Connection.this.requestObject); } });
	break;""", '\t' * 7)

READ_RELATED_LIST_RESPONSE = indent("""
case REQUEST_READ_{{ parent_name|upper }}_{{ name_plural|upper }}:
	jsonArray = new JSONArray(response);
	responseArray = new {{ name }}[jsonArray.length()];
	for(int i = 0; i < jsonArray.length(); ++i) {% templatetag openbrace %}{% if subclasses %}
		JSONObject jsonObject = jsonArray.getJSONObject(i);{% for subclass in subclasses %}
		{% if forloop.counter0 %}else {% endif %}if(jsonObject.has("{{ subclass.0 }}"))
			responseArray[i] = new {{ subclass.1 }}(jsonObject);{% endfor %}{% else %}
		responseArray[i] = new {{ name }}(jsonArray.getJSONObject(i));{% endif %}
	}
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidRead{{ name_plural }}For{{ parent_name }}({{ app_name }}Connection.this, ({{ name }}[])responseArray); } });
	break;""", '\t' * 7)

CREATE_RELATED_RESPONSE = indent("""
case REQUEST_CREATE_{{ parent_name|upper }}_{{ name|upper }}:
	(({{ name }}){{ app_name }}Connection.this.requestObject).setObjectId(API.parseInt({{ app_name }}Connection.this.connection.getHeaderField(XHEADER_NEW_OBJECT_ID)));
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidCreate{{ name }}For{{ parent_name }}({{ app_name }}Connection.this, ({{ name }}){{ app_name }}Connection.this.requestObject); } });
	break;""", '\t' * 7)

READ_RELATED_OBJECT_RESPONSE = indent("""
case REQUEST_READ_{{ parent_name|upper }}_{{ name|upper }}:
	responseObject = new {{ name }}(new JSONObject(response));
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidRead{{ name }}For{{ parent_name }}({{ app_name }}Connection.this, ({{ name }})responseObject); } });
	break;""", '\t' * 7)

UPDATE_RELATED_RESPONSE = indent("""
case REQUEST_UPDATE_{{ parent_name|upper }}_{{ name|upper }}:
	if({{ app_name }}Connection.this.listener != null)
		API.runOnUiThread(new Runnable() { public void run() { {{ app_name }}Connection.this.listener.{{ app_name_lower }}ConnectionDidUpdate{{ name }}For{{ parent_name }}({{ app_name }}Connection.this, ({{ name }}){{ app_name }}Connection.this.requestObject); } });
	break;""", '\t' * 7)

class Command(BaseCommand, GenerateConnectionsCommand):
	help = 'Generate Android connections for API endpoints.'
	option_list = BaseCommand.option_list + GenerateConnectionsCommand.option_list + (
		make_option('--connection',
			type='string',
			dest='connection',
			default='APIURLConnection',
			help='Connection class to use that should have the same interface as ???.'),
		make_option('--package',
			action='store',
			dest='package',
			type='string',
			default='com.business.app',
			help='Set the Android package name.'),
	)

	def extra_context(self):
		return {'package': self.package, 'connection': self.connection}

	def handle(self, *args, **options):
		# Remember options
		self.package = options['package']
		self.connection = options['connection']

		# Create the templates
		self.templates = []
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'android-connection.java')
		with open(template_path) as f:
			self.templates.append(Template(f.read()))
		self.template_extensions = ['java']

		# Create the mappings
		request_types_base = 'REQUEST_%s_%s'
		request_types_mapping = {
			'READ_OBJECT': Template(request_types_base % ('READ', '{{ name|upper }}')),
			'READ_LIST': Template(request_types_base % ('READ', '{{ name_plural|upper }}')),
			'CREATE': Template(request_types_base % ('CREATE', '{{ name|upper }}')),
			'UPDATE': Template(request_types_base % ('UPDATE', '{{ name|upper }}')),
			'DELETE': Template(request_types_base % ('DELETE', '{{ name|upper }}')),
			'READ_RELATED_LIST': Template(request_types_base % ('READ', '{{ parent_name|upper }}_{{ name_plural|upper }}')),
			'CREATE_RELATED': Template(request_types_base % ('CREATE', '{{ parent_name|upper }}_{{ name|upper }}')),
			'READ_RELATED_OBJECT': Template(request_types_base % ('READ', '{{ parent_name|upper }}_{{ name|upper }}')),
			'UPDATE_RELATED': Template(request_types_base % ('UPDATE', '{{ parent_name|upper }}_{{ name|upper }}')),
			'RemoveDuplicates': True,
			'Sort': True
		}

		urls_mapping = {}
		urls_mapping['READ_OBJECT'] = Template('public static final String URL_{{ name|upper }}{% if param %}_WITH_{{ param|upper }}{% endif %} = "{{ url_format }}";')
		urls_mapping['READ_LIST'] = urls_mapping['CREATE'] = Template('public static final String URL_{{ name_plural|upper }} = "{{ url_format }}";')
		urls_mapping['UPDATE'] = urls_mapping['DELETE'] = Template('public static final String URL_{{ name|upper }} = "{{ url_format }}";')
		urls_mapping['READ_RELATED_LIST'] = urls_mapping['CREATE_RELATED'] = Template('public static final String URL_{{ parent_name|upper }}_{{ name_plural|upper }}{% if param %}_WITH_{{ param|upper }}{% endif %} = "{{ url_format }}";')
		urls_mapping['READ_RELATED_OBJECT'] = urls_mapping['UPDATE_RELATED'] = Template('public static final String URL_{{ parent_name|upper }}_{{ name|upper }}{% if param %}_WITH_{{ param|upper }}{% endif %}  = "{{ url_format }}";')
		urls_mapping['RemoveDuplicates'] = True

		listener_method_decls_base = 'public void {{ app_name_lower }}ConnectionDid%s({{ app_name }}Connection connection, %s);'
		listener_method_decls_mapping = {
			'READ_OBJECT': Template(listener_method_decls_base % ('Read{{ name }}', '{{ name }} {{ name_lower }}')),
			'READ_LIST': Template(listener_method_decls_base % ('Read{{ name_plural }}', '{{ name }}[] {{ name_plural_lower }}')),
			'CREATE': Template(listener_method_decls_base % ('Create{{ name }}', '{{ name }} {{ name_lower }}')),
			'UPDATE': Template(listener_method_decls_base % ('Update{{ name }}', '{{ name }} {{ name_lower }}')),
			'DELETE': Template(listener_method_decls_base % ('Delete{{ name }}', '{{ name }} {{ name_lower }}')),
			'READ_RELATED_LIST': Template(listener_method_decls_base % ('Read{{ name_plural }}For{{ parent_name }}', '{{ name }}[] {{ name_plural_lower }}')),
			'CREATE_RELATED': Template(listener_method_decls_base % ('Create{{ name }}For{{ parent_name }}', '{{ name }} {{ name_lower }}')),
			'READ_RELATED_OBJECT': Template(listener_method_decls_base % ('Read{{ name }}For{{ parent_name }}', '{{ name }} {{ name_lower }}')),
			'UPDATE_RELATED': Template(listener_method_decls_base % ('Update{{ name }}For{{ parent_name }}', '{{ name }} {{ name_lower }}')),
			'RemoveDuplicates': True,
			'Sort': True
		}

		basic_listener_methods_base = 'public void {{ app_name_lower }}ConnectionDid%s({{ app_name }}Connection connection, %s) { Log.i(API.TAG, "{{ app_name }} did %s."); }'
		basic_listener_methods_mapping = {
			'READ_OBJECT': Template(basic_listener_methods_base % ('Read{{ name }}', '{{ name }} {{ name_lower }}', 'read {{ name_lower }}')),
			'READ_LIST': Template(basic_listener_methods_base % ('Read{{ name_plural }}', '{{ name }}[] {{ name_plural_lower }}', 'read " + Integer.toString({{ name_plural_lower }}.length) + " {{ name_plural_lower }}')),
			'CREATE': Template(basic_listener_methods_base % ('Create{{ name }}', '{{ name }} {{ name_lower }}', 'create {{ name_lower }}')),
			'UPDATE': Template(basic_listener_methods_base % ('Update{{ name }}', '{{ name }} {{ name_lower }}', 'update {{ name_lower }}')),
			'DELETE': Template(basic_listener_methods_base % ('Delete{{ name }}', '{{ name }} {{ name_lower }}', 'delete {{ name_lower }}')),
			'READ_RELATED_LIST': Template(basic_listener_methods_base % ('Read{{ name_plural }}For{{ parent_name }}', '{{ name }}[] {{ name_plural_lower }}', 'read " + Integer.toString({{ name_plural_lower }}.length) + " {{ name_plural_lower }} for {{ parent_name_lower }}')),
			'CREATE_RELATED': Template(basic_listener_methods_base % ('Create{{ name }}For{{ parent_name }}', '{{ name }} {{ name_lower }}', 'create {{ name_lower }} for {{ parent_name_lower }}')),
			'READ_RELATED_OBJECT': Template(basic_listener_methods_base % ('Read{{ name }}For{{ parent_name }}', '{{ name }} {{ name_lower }}', 'read {{ name_lower }} for {{ parent_name_lower }}')),
			'UPDATE_RELATED': Template(basic_listener_methods_base % ('Update{{ name }}For{{ parent_name }}', '{{ name }} {{ name_lower }}', 'update {{ name_lower }} for {{ parent_name_lower }}')),
			'RemoveDuplicates': True,
			'Sort': True
		}

		methods_mapping = {
			'READ_OBJECT': Template(READ_OBJECT_METHOD),
			'READ_LIST': Template(READ_LIST_METHOD),
			'CREATE': Template(CREATE_METHOD),
			'UPDATE': Template(UPDATE_METHOD),
			'DELETE': Template(DELETE_METHOD),
			'READ_RELATED_LIST': Template(READ_RELATED_LIST_METHOD),
			'CREATE_RELATED': Template(CREATE_RELATED_METHOD),
			'READ_RELATED_OBJECT': Template(READ_RELATED_OBJECT_METHOD),
			'UPDATE_RELATED': Template(UPDATE_RELATED_METHOD)
		}

		response_cases_mapping = {
			'READ_OBJECT': Template(READ_OBJECT_RESPONSE),
			'READ_LIST': Template(READ_LIST_RESPONSE),
			'CREATE': Template(CREATE_RESPONSE),
			'UPDATE': Template(UPDATE_RESPONSE),
			'DELETE': Template(DELETE_RESPONSE),
			'READ_RELATED_LIST': Template(READ_RELATED_LIST_RESPONSE),
			'CREATE_RELATED': Template(CREATE_RELATED_RESPONSE),
			'READ_RELATED_OBJECT': Template(READ_RELATED_OBJECT_RESPONSE),
			'UPDATE_RELATED': Template(UPDATE_RELATED_RESPONSE),
			'RemoveDuplicates': True
		}

		# Set the configuration options and render
		self.mappings = {'urls': urls_mapping, 'request_types': request_types_mapping, 'listener_method_decls': listener_method_decls_mapping, 'basic_listener_methods': basic_listener_methods_mapping, 'methods': methods_mapping, 'response_cases': response_cases_mapping}
		self.object_id_format = "%d"
		self.slug_format = "%s"
		self.object_id_type = "int"
		self.slug_type = "String"
		self.render(*args, **options)



import ast
import os
from optparse import make_option

from django.core.management.base import BaseCommand
from django.template import Template

from symmetric.management.functions import get_api_properties
from symmetric.management.generatemodels import GenerateModelsCommand

class ApiPropertyGetMethodTransformer(ast.NodeTransformer):
	"""Renames uses of other api properties into get method calls e.g. getProperty()."""
	def __init__(self, model):
		self.api_properties = get_api_properties(model)

	def visit_Attribute(self, node):
		if type(node.value) is ast.Name and node.value.id == 'self' and type(node.attr) is str and node.attr in self.api_properties:
			# underscore here because it will later be converted to camelCase
			node.attr = 'get_%s()' % node.attr
		return node

class Command(BaseCommand, GenerateModelsCommand):
	help = 'Generate Android models for API models or endpoints.'
	option_list = BaseCommand.option_list + GenerateModelsCommand.option_list + (
		make_option('--package',
			action='store',
			dest='package',
			type='string',
			default='com.business.app',
			help='Set the Android package name.'),
	)

	def extra_context(self, model, api_model):
		# first: random.seed()
		# uid = '1' + ''.join([str(random.randint(0,9)) for x in range(18)])
		return {'package': self.package, 'uid': model.__name__.__hash__()}

	def handle(self, *args, **options):
		# Remember options
		self.package = options['package']

		# Create the template
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'android-model.java')
		with open(template_path) as f:
			self.templates = [Template(f.read())]
		self.template_extensions = ['java']

		# Create the mappings
		ivar_base = '{{name}}{% if default %} = {{default}}{% endif %};'
		ivar_base_str = '{{name}}{% if default %} = "{{default}}"{% endif %};'
		ivar_mapping = {
			'AutoField': Template('public int ' + ivar_base),
			'CharField': Template('public String ' + ivar_base_str),
			'IntegerField': Template('public int ' + ivar_base),
			'BigIntegerField': Template('public long ' + ivar_base),
			'DateField': Template('public Date ' + ivar_base),
			'DecimalField': Template('public float ' + ivar_base),
			'BooleanField': Template('public boolean {{name}}{% if default %} = true{% endif %};'),
			'ForeignKey': Template('public {% if included %}{{included_name}}{% else %}int{% endif %} {{name}};'),
			'JSONField': Template('public JSONObject ' + ivar_base),
			'ArrayField': Template('public JSONArray ' + ivar_base),
			'Field': 'public int {name};'
		}
		read_json_base = 'this.{{name}} = jsonObject.opt%s("{{name}}");'
		read_json_mapping = {
			'AutoField': Template(read_json_base % 'Int'),
			'CharField': Template('this.{{name}} = (jsonObject.isNull("{{name}}")) ? null : jsonObject.optString("{{name}}");'),
			'IntegerField': Template(read_json_base % 'Int'),
			'BigIntegerField': Template(read_json_base % 'Long'),
			'DateField': Template('this.{{name}} = (jsonObject.isNull("{{name}}")) ? null : API.parseDate(jsonObject.optString("{{name}}"));'),
			'DecimalField': Template(read_json_base % 'Double'),
			'BooleanField': Template(read_json_base % 'Boolean'),
			'ForeignKey': Template('this.{{name}} = {% if included %}(jsonObject.isNull("{{name}}")) ? null : new {{included_name}}(jsonObject.optJSONObject("{{name}}")){% else %}jsonObject.optInt("{{name}}"){% endif %};'),
			'JSONField': Template(read_json_base % 'JSONObject'),
			'ArrayField': Template(read_json_base % 'JSONArray'),
			'Field': Template(read_json_base % 'Int')
		}
		write_json_mapping = {
			'DateField': Template('jsonObject.put("{{name}}", {% if null %}(this.{{name}} == null) ? JSONObject.NULL : {% endif %}API.formatDate(this.{{name}}));'),
			'ForeignKey': Template('jsonObject.put("{{name}}", {% if included %}{% if included_readonly %}(this.{{included_obj_name}} != null) ? this.{{included_obj_name}}.getObjectId() : JSONObject.NULL{% else %}(this.{{name}} == null) ? JSONObject.NULL : this.{{name}}.getJSONObject(){% endif %}{% else %}(this.{{name}} == 0) ? JSONObject.NULL : this.{{name}}{% endif %});'),
			'Field': Template('jsonObject.put("{{name}}", {% if null %}(this.{{name}} == null) ? JSONObject.NULL : {% endif %}this.{{name}});'),
			'WriteOnly': True
		}
		read_parcel_base = 'this.{{name}} = in.read%s();'
		read_parcel_mapping = {
			'AutoField': Template(read_parcel_base % 'Int'),
			'CharField': Template(read_parcel_base % 'String'),
			'IntegerField': Template(read_parcel_base % 'Int'),
			'BigIntegerField': Template(read_parcel_base % 'Long'),
			'DateField': Template('this.{{name}} = (Date)in.readSerializable();'),
			'DecimalField': Template(read_parcel_base % 'Double'),
			'BooleanField': 'boolean[] _{name} = new boolean[1];\nin.readBooleanArray(_{name});\nthis.{name} = _{name}[0];',
			'ForeignKey': Template('this.{{name}} = {% if included %}({{included_name}})in.readSerializable(){% else %}in.readInt(){% endif %};'),
			'JSONField': Template('try { this.{{name}} = new JSONObject(in.readString()); } catch(JSONException e) { }'),
			'ArrayField': Template('try { this.{{name}} = new JSONArray(in.readString()); } catch(JSONException e) { }'),
			'Field': Template(read_parcel_base % 'Int')
		}
		write_parcel_base = 'dest.write%s(this.{{name}});'
		write_parcel_mapping = {
			'AutoField': Template(write_parcel_base % 'Int'),
			'CharField': Template(write_parcel_base % 'String'),
			'IntegerField': Template(write_parcel_base % 'Int'),
			'BigIntegerField': Template(write_parcel_base % 'Long'),
			'DateField': Template(write_parcel_base % 'Serializable'),
			'DecimalField': Template(write_parcel_base % 'Double'),
			'BooleanField': 'dest.writeBooleanArray(new boolean[] {{ this.{name} }});',
			'ForeignKey': Template(write_parcel_base % '{% if included %}Serializable{% else %}Int{% endif %}'),
			'JSONField': Template('dest.writeString(this.{{name}}.toString());'),
			'ArrayField': Template('dest.writeString(this.{{name}}.toString());'),
			'Field': Template(write_parcel_base % 'Int')
		}
		read_external_base = 'this.{{name}} = %sinput.read%s();'
		read_external_mapping = {
			'AutoField': Template(read_external_base % ('', 'Int')),
			'CharField': Template(read_external_base % ('(String)', 'Object')),
			'IntegerField': Template(read_external_base % ('', 'Int')),
			'BigIntegerField': Template(read_external_base % ('', 'Long')),
			'DateField': Template(read_external_base % ('(Date)', 'Object')),
			'DecimalField': Template(read_external_base % ('', 'Float')),
			'BooleanField': Template(read_external_base % ('', 'Boolean')),
			'ForeignKey': Template(read_external_base % ('{% if included %}({{included_name}}){% endif %}', '{% if included %}Object{% else %}Int{% endif %}')),
			'JSONField': Template('try { this.{{name}} = new JSONObject((String)input.readObject()); } catch(JSONException e) { }'),
			'ArrayField': Template('try { this.{{name}} = new JSONArray((String)input.readObject()); } catch(JSONException e) { }'),
			'Field': Template(read_external_base % ('', 'Int'))
		}
		write_external_base = 'output.write%s(this.{{name}});'
		write_external_mapping = {
			'AutoField': Template(write_external_base % 'Int'),
			'CharField': Template(write_external_base % 'Object'),
			'IntegerField': Template(write_external_base % 'Int'),
			'BigIntegerField': Template(write_external_base % 'Long'),
			'DateField': Template(write_external_base % 'Object'),
			'DecimalField': Template(write_external_base % 'Float'),
			'BooleanField': Template(write_external_base % 'Boolean'),
			'ForeignKey': Template(write_external_base % '{% if included %}Object{% else %}Int{% endif %}'),
			'JSONField': Template('output.writeObject(this.{{name}}.toString());'),
			'ArrayField': Template('output.writeObject(this.{{name}}.toString());'),
			'Field': Template(write_external_base % 'Int')
		}
		self.lang = 'java'
		self.property_transformer = ApiPropertyGetMethodTransformer
		self.property_types = {int: 'int', float: 'float', str: 'String', bool: 'boolean'}
		self.property_implementations = '\tpublic {type} get{name_upper}()\n\t{{\n\t\treturn {code};\n\t}}\n'
		self.mappings = {'ivars': ivar_mapping, 'read_json': read_json_mapping, 'write_json': write_json_mapping, 'read_parcel': read_parcel_mapping, 'write_parcel': write_parcel_mapping, 'read_external': read_external_mapping, 'write_external': write_external_mapping }
		self.expand_mappings('CharField', 'TextField', 'IPAddressField', 'GenericIPAddressField')
		self.render(*args, **options)



import datetime

from django.conf import settings
from django.db import models
from django.utils import timezone

def underscore_to_camel_case(string):
	words = [word.capitalize() for word in string.split('_')]
	words[0] = words[0].lower()
	return ''.join(words)

def camel_case_to_underscore(string):
	words = []
	start_index = 0
	for index, c in enumerate(string):
		# Ignore the first character regardless of case
		if c.isupper() and index:
			words.append(string[start_index:index].lower())
			start_index = index
	words.append(string[start_index:].lower())
	return '_'.join(words)

def sanitize_order_by(string):
	"""Make sure the string has no double underscores, also convert from camelcase."""
	if string and string.find('__') == -1 and string.find('?') == -1:
		return camel_case_to_underscore(string)
	return ''

def iso_8601_to_time(iso):
	"""Parse an iso 8601 date into a datetime.time."""
	if not iso:
		return None
	return datetime.datetime.strptime(iso, '%H:%M:%S').time()

def iso_8601_to_date(iso):
	"""Parse an iso 8601 date into a datetime.date."""
	if not iso:
		return None
	return datetime.datetime.strptime(iso[:10], '%Y-%m-%d').date()

def iso_8601_to_datetime(iso):
	"""Parse an iso 8601 string into a timezone aware datetime, ignoring and fractional seconds."""
	if not iso:
		return None
	dt = datetime.datetime.strptime(iso[:19], '%Y-%m-%dT%H:%M:%S')
	# strptime doesn't seem to handle timezones, parse them here
	if len(iso) == 19:
		return timezone.make_aware(dt, timezone.get_current_timezone())
	else:
		# Make the datetime UTC if Z is the timezone, ignoring fractional seconds in between
		if (len(iso) == 20 or iso[19] == '.') and iso[-1] == 'Z':
			return timezone.make_aware(dt, timezone.UTC())
		# Parse a complete timezone e.g. +00:00, checking for the correct length or ignored fractional seconds
		if (len(iso) == 25 or iso[19] == '.') and iso[-6] in ('+', '-') and iso[-3] == ':':
			try:
				hours = int(iso[-5:-3])
				minutes = int(iso[-2:])
				minutes += hours * 60
				if iso[-6] == '-':
					minutes = -minutes
				return timezone.make_aware(dt, timezone.get_fixed_timezone(minutes))
			except:
				# drop through and raise the exception
				pass
		raise ValueError('Invalid timezone %s.' % iso[19:])

def time_to_iso_8601(t):
	"""Format a datetime.time as an iso 8601 string - HH:MM:SS."""
	if t:
		return t.replace(microsecond=0).isoformat()
	else:
		return None

def date_to_iso_8601(d):
	"""Format a datetime.date as an iso 8601 string - YYYY-MM-DD."""
	if d:
		return d.isoformat()
	else:
		return None

def datetime_to_iso_8601(dt):
	"""Format a datetime as an iso 8601 string - YYYY-MM-DDTHH:MM:SS with optional timezone +HH:MM."""
	if dt:
		return dt.replace(microsecond=0).isoformat()
	else:
		return None

def decode_int(value):
	"""Decode an int after checking to make sure it is not already a int, 0.0, or empty."""
	if isinstance(value, (int, long)):
		return value
	elif value == 0.0:
		return 0
	elif not value:
		return None
	return int(value)

def decode_float(value):
	"""Decode a float after checking to make sure it is not already a float, 0, or empty."""
	if type(value) is float:
		return value
	elif value == 0:
		return 0.0
	elif not value:
		return None
	return float(value)

def decode_bool(value):
	"""Decode a bool after checking to make sure it is not already a bool, int, or empty.

	If value is 0.0, None is returned because floats shouldn't be used.
	"""
	t = type(value)
	if t is bool:
		return value
	elif t is int:
		return bool(value)
	elif not value:
		return None
	elif t in (str, unicode) and value.lower() in ('false', 'f', 'off', 'no'):
		return False
	return True

_api_models = {}

class _ApiModel(object):
	def __init__(self, model):
		# Tuples of (name, encoded_name, encode, decode)
		self.fields = []
		self.list_fields = []
		self.encoded_fields = {}
		self.id_field = None
		self.select_related_args = []
		# data dictionary, set fields instead of creating a new dictionary for each get_data
		self._data = {}
		self._list_data = {}
		include_fields = None
		exclude_fields = None
		include_related = ()
		list_fields = None
		update_fields = None
		readonly_fields = None
		camelcase = getattr(settings, 'API_CAMELCASE', True)
		renameid = getattr(settings, 'API_RENAME_ID', True)
		if hasattr(model, 'API'):
			if hasattr(model.API, 'include_fields'):
				include_fields = model.API.include_fields
			if hasattr(model.API, 'exclude_fields'):
				exclude_fields = model.API.exclude_fields
			if hasattr(model.API, 'include_related'):
				include_related = model.API.include_related
			if hasattr(model.API, 'list_fields'):
				list_fields = model.API.list_fields
			if hasattr(model.API, 'update_fields'):
				update_fields = model.API.update_fields
			if hasattr(model.API, 'readonly_fields'):
				readonly_fields = model.API.readonly_fields

		# Calculate all of the fields and list fields
		for field in model._meta.fields:
			if include_fields and field.name not in include_fields:
				continue
			elif exclude_fields and field.name in exclude_fields:
				continue
			else:
				name = field.name
				if field.name == 'id' and renameid:
					encoded_name = camel_case_to_underscore(model.__name__) + '_id'
				else:
					encoded_name = field.name
				if isinstance(field, models.IntegerField):
					encode = None
					decode = decode_int
				elif isinstance(field, models.FloatField):
					encode = None
					decode = decode_float
				elif isinstance(field, models.BooleanField):
					encode = None
					decode = decode_bool
				elif isinstance(field, models.DateTimeField):
					encode = datetime_to_iso_8601
					decode = iso_8601_to_datetime
				elif isinstance(field, models.TimeField):
					encode = time_to_iso_8601
					decode = iso_8601_to_time
				elif isinstance(field, models.DateField):
					encode = date_to_iso_8601
					decode = iso_8601_to_date
				elif isinstance(field, models.ForeignKey):
					if field.name in include_related:
						encode = get_object_data
						decode = set_object_data
						# Calculate the select_related_args
						related_model = _get_api_model(field.rel.to)
						if related_model.select_related_args:
							for arg in related_model.select_related_args:
								self.select_related_args.append('%s__%s' % (field.name, arg))
						else:
							self.select_related_args.append(field.name)
						# For include related fields, also add an encoded_field entry for the option of updating the foreign key to another entry
						# Setting the name_id attribute to None has the same effect as setting name to None, it will set the foreign key to null in the db
						field_coding = (name + '_id', underscore_to_camel_case(encoded_name + '_id') if camelcase else encoded_name + '_id', None, decode_int)
						self.encoded_fields[field_coding[1]] = field_coding
					else:
						name += '_id'
						encoded_name += '_id'
						encode = None
						decode = decode_int
						# For pointers to parent models, add as readonly field without the ptr suffix
						if isinstance(field, models.OneToOneField) and encoded_name.endswith('_ptr_id'):
							encoded_name = encoded_name[:-7] + '_id'
							field_coding = (name, underscore_to_camel_case(encoded_name) if camelcase else encoded_name, None, decode_int)
							self.fields.append(field_coding)
							self.list_fields.append(field_coding)
							continue
				elif isinstance(field, models.AutoField):
					encode = None
					decode = decode_int
				elif isinstance(field, (models.FileField, models.ManyToManyField)):
					continue
				else:
					encode = None
					decode = None

				if camelcase:
					field_coding = (name, underscore_to_camel_case(encoded_name), encode, decode)
				else:
					field_coding = (name, encoded_name, encode, decode)
				self.fields.append(field_coding)
				if field.name == 'id':
					self.id_field = field_coding
				if not field.editable or field.primary_key:
					pass
				elif update_fields and field.name not in update_fields:
					pass
				elif readonly_fields and field.name in readonly_fields:
					pass
				else:
					self.encoded_fields[field_coding[1]] = field_coding
				if list_fields is None or field.name in list_fields:
					if field_coding[2] == get_object_data:
						self.list_fields.append((field_coding[0], field_coding[1], get_object_list_data, field_coding[3]))
					else:
						self.list_fields.append(field_coding)

	def get_list_data(self, obj):
		for name, encoded_name, encode, decode in self.list_fields:
			if encode:
				self._list_data[encoded_name] = encode(getattr(obj, name))
			else:
				self._list_data[encoded_name] = getattr(obj, name)
		return self._list_data

	def get_data(self, obj):
		for name, encoded_name, encode, decode in self.fields:
			if encode:
				self._data[encoded_name] = encode(getattr(obj, name))
			else:
				self._data[encoded_name] = getattr(obj, name)
		return self._data

	def set_data(self, obj, data):
		for key, value in data.iteritems():
			if self.encoded_fields.has_key(key):
				name, encoded_name, encode, decode = self.encoded_fields[key]
				if decode:
					if decode is set_object_data:
						decode(getattr(obj, name), value)
					else:
						setattr(obj, name, decode(value))
				else:
					setattr(obj, name, value)

def _get_api_model(model):
	key = model.__module__ + model.__name__
	if not _api_models.has_key(key):
		_api_models[key] = _ApiModel(model)
	return _api_models[key]

def get_object_list_data(obj):
	if obj is None:
		return None
	model = _get_api_model(type(obj))
	return model.get_list_data(obj)

def get_object_data(obj):
	if obj is None:
		return None
	model = _get_api_model(type(obj))
	data = model.get_data(obj)
	if hasattr(obj, '_exclude_data'):
		# Return a copy with specific data fields removed
		data = dict(data)
		for excluded in obj._exclude_data:
			del data[excluded]
	return data

def set_object_data(obj, data):
	model = _get_api_model(type(obj))
	model.set_data(obj, data)

def save_object(obj):
	model = type(obj)
	if hasattr(model, 'API') and hasattr(model.API, 'include_related'):
		for field in model.API.include_related:
			# Do a quick check for readonly status - this is for a slight performance boost only
			# There are other settings that can cause the sub-object to be readonly, but it's not enforced here
			if hasattr(model.API, 'readonly_fields') and field in model.API.readonly_fields:
				continue
			subobj = getattr(obj, field, None)
			if subobj:
				save_object(subobj)
	obj.full_clean()
	obj.save()



import ast

from django.conf import settings

from symmetric.functions import underscore_to_camel_case

default_token_mapping = {
	ast.Add: '+', ast.Sub: '-', ast.Mult: '*', ast.Div: '/', ast.Mod: '%', ast.LShift: '<<', ast.RShift: '>>', ast.BitOr: '|', ast.BitXor: '^', ast.BitAnd: '&', ast.And: '&&', ast.Or: '||',
	ast.Eq: '==', ast.NotEq: '!=', ast.Lt: '<', ast.LtE: '<=', ast.Gt: '>', ast.GtE: '>=', ast.Is: '==', ast.IsNot: '!=',
	ast.Invert: '~', ast.Not: '!', ast.UAdd: '+', ast.USub: '-',

	ast.Attribute: ('','.'),
	ast.Str: ('"', '"'),
	ast.BinOp: ('(', ' ', ' ', ')'),
	ast.UnaryOp: ('', '(', ')'),
	ast.Compare: ('', ' ', ' '),
	ast.IfExp: ('(', ' ? ', ' : ', ')'),
	ast.Subscript: ('', '.substring(', ')'),
	ast.Slice: ('',', ')
}

js_token_mapping = default_token_mapping.copy()
js_token_mapping.update({ast.Eq: '===', ast.NotEq: '!==', ast.Is: '===', ast.IsNot: '!=='})

objc_token_mapping = default_token_mapping.copy()
objc_token_mapping.update({ast.Str: ('@"', '"'), ast.Subscript: ('[', ' substringWithRange:NSMakeRange(', ')]')})

# Bridge to objc, because substring method in swift is too complicated: http://stackoverflow.com/questions/39677330/how-does-string-substring-work-in-swift-3
swift_token_mapping = default_token_mapping.copy()
swift_token_mapping.update({ast.Subscript: ('(', ' as NSString).substring(with: NSMakeRange(', '))')})

def translate_ast(node, token_mapping=default_token_mapping):
	tokens = token_mapping.get(type(node))
	if not tokens:
		if isinstance(node, ast.BoolOp):
			code = [translate_ast(value, token_mapping) for value in node.values]
			return '(%s)' % (' %s ' % translate_ast(node.op, token_mapping)).join(code)
		elif isinstance(node, ast.expr_context):
			# Nothing for an expr_context, like Load, Store, Del, when processing an ast.Subscript
			return ''
		elif isinstance(node, ast.AST):
			value = getattr(node, node._fields[0])
			return translate_ast(value, token_mapping)
		else:
			return str(node)
	elif isinstance(tokens, (str, unicode)):
		return tokens
	else:
		code = []
		for i, token in enumerate(tokens):
			code.append(token)
			if i < len(node._fields):
				exp = getattr(node, node._fields[i], None)
				if exp:
					code.append(translate_ast(exp, token_mapping))
		return ''.join(code)

class KeywordTransformer(ast.NodeTransformer):
	"""Rename self to this. Also rename None, True, and False."""
	def __init__(self, lang):
		if lang == 'objc':
			self.id_self = 'self'
			self.id_none = 'nil'
			self.id_true = 'YES'
			self.id_false = 'NO'
		elif lang == 'swift':
			self.id_self = 'self'
			self.id_none = 'nil'
			self.id_true = 'true'
			self.id_false = 'false'
		else:
			self.id_self = 'this'
			self.id_none = 'null'
			self.id_true = 'true'
			self.id_false = 'false'
		if lang == 'js' or lang == 'es6':
			self.id_none = 'void(0)'

	def visit_Name(self, node):
		name = node.id.lower()
		if name == 'self':
			node.id = self.id_self
		elif name == 'none':
			node.id = self.id_none
		elif name == 'true':
			node.id = self.id_true
		elif name == 'false':
			node.id = self.id_false
		return node

class CamelcaseTransformer(ast.NodeTransformer):
	"""Convert all names to camelcase."""
	def visit_Name(self, node):
		node.id = underscore_to_camel_case(node.id)
		return node

	def visit_Attribute(self, node):
		# Some attributes, such as those for an objdict on a JSONField might already be in camelcase, so check for an underscore first
		if node.attr.find('_') != -1:
			node.attr = underscore_to_camel_case(node.attr)
		return node

class IdTransformer(ast.NodeTransformer):
	"""Convert all id attributes."""
	def __init__(self, lang):
		self.lang = lang

	def visit_Attribute(self, node):
		if type(node.attr) is str and node.attr == 'id':
			if self.lang == 'objc':
				node.attr = 'objectId'
			elif self.lang == 'java':
				node.attr = 'getObjectId()'
		return node

class CompareTransformer(ast.NodeTransformer):
	"""Convert the Compare's fields to a single comparison, not compound."""
	def visit_Compare(self, node):
		node.ops = node.ops[0]
		node.comparators = node.comparators[0]
		return node

def _has_str_node(node):
	class _Visitor(ast.NodeVisitor):
		def visit_Str(self, node):
			self.found = True
	visitor = _Visitor()
	visitor.found = False
	visitor.visit(node)
	return visitor.found

class StringFormatTransformer(ast.NodeTransformer):
	"""Translate a string format operation (%) directly to desired language."""
	def __init__(self, lang):
		self.lang = lang

	def visit_BinOp(self, node):
		"""
		Visit for each binary operator. Formatting is determined by a Str node appearing somewhere in the left child.
		The left child may not always be a string but an expression that results in a string, so it is first transformed and translated.

		NOTE:
		The format string must be a string and not a variable containing a format string.
		Also, translating to Javascript currently doesn't support the %% format that resolves into the single % character.
		"""
		if type(node.op) is ast.Mod and _has_str_node(node.left):
			left = self.visit(node.left)
			if isinstance(node.right, (ast.Str, ast.Num, ast.Attribute, ast.Name, ast.Subscript)):
				elts = [node.right]
			else:
				elts = node.right.elts
			if self.lang == 'js':
				format = translate_ast(left, js_token_mapping)
				for elt in elts:
					format += ".replace(/%%[sdfg]/, %s)" % translate_ast(elt, js_token_mapping)
			elif self.lang == 'es6':
				format = translate_ast(left, js_token_mapping)
				for elt in elts:
					idx = format.find('%')
					percentf = format[idx:idx+2]
					if percentf == '%%':
						format = format.replace(percentf, '\u{25}', 1)
					else:
						format = format.replace(percentf, '${%s}' % translate_ast(elt, js_token_mapping), 1)
				format = '`%s`' % format[1:-1]
			elif self.lang == 'objc':
				format = '[NSString stringWithFormat:%s' % translate_ast(left, objc_token_mapping).replace("%s", "%@")
				for elt in elts:
					format += ", %s" % translate_ast(elt, objc_token_mapping)
				format += ']'
			elif self.lang == 'swift':
				format = 'String(format:%s' % translate_ast(left, swift_token_mapping).replace("%s", "%@")
				for elt in elts:
					format += ", %s" % translate_ast(elt, swift_token_mapping)
				format += ')'
			elif self.lang == 'java':
				format = 'String.format(%s' % translate_ast(left, default_token_mapping)
				for elt in elts:
					format += ", %s" % translate_ast(elt, default_token_mapping)
				format += ')'
			return format
		else:
			return node

class SliceTransformer(ast.NodeTransformer):
	"""Objective-C indexing works with location, length not start, end index."""
	def __init__(self, lang):
		self.lang = lang

	def visit_Slice(self, node):
		if (self.lang == 'objc' or self.lang == 'swift') and node.upper:
			node.upper = ast.BinOp(node.upper, ast.Sub(), node.lower)
		return node

class CastTransformer(ast.NodeTransformer):
	"""Convert the float, int, and long function calls into a cast unary operation. Convert the len function call into string length."""
	def __init__(self, lang):
		self.lang = lang

	def visit_Call(self, node):
		if node.func.id == 'len':
			if self.lang == 'java':
				return ast.Attribute(node.args[0], ast.Name('length()', ast.Load()), ast.Load())
			else:
				return ast.Attribute(node.args[0], ast.Name('length', ast.Load()), ast.Load())
		else:
			if self.lang == 'js' or lang == 'es6':
				return node.args[0]
			else:
				return ast.UnaryOp('(%s)' % node.func.id, node.args[0])

def translate_code(code, lang='js', transformer=None):
	node = ast.parse(code, mode='eval')
	# Apply a bunch of transformers, note the sequence of these is important, especially for the StringFormatTransformer being last
	if transformer:
		node = transformer.visit(node)
	node = CompareTransformer().visit(node)
	if getattr(settings, 'API_CAMELCASE', True):
		node = CamelcaseTransformer().visit(node)
	if getattr(settings, 'API_RENAME_ID', True):
		node = IdTransformer(lang).visit(node)
	node = KeywordTransformer(lang).visit(node)
	node = SliceTransformer(lang).visit(node)
	node = CastTransformer(lang).visit(node)
	node = StringFormatTransformer(lang).visit(node)
	# Choose a token mapping and then perform the translation
	token_mapping = default_token_mapping
	if lang == 'js' or lang == 'es6':
		token_mapping = js_token_mapping
	elif lang == 'objc':
		token_mapping = objc_token_mapping
	elif lang == 'swift':
		token_mapping = swift_token_mapping
	return translate_ast(node, token_mapping)

def data_to_objc(data, mutable=True):
	objc = ''
	if callable(data):
		data = data()
	if data is None:
		return 'nil'
	if isinstance(data, (str, unicode)):
		return '@"%s"' % data.replace('"', '\\"')
	elif isinstance(data, bool):
		return '@YES' if data else '@NO'
	elif isinstance(data, (int, float)):
		return '@%s' % data
	elif isinstance(data, (tuple, list, set)):
		objc = '@[%s]' % ', '.join([data_to_objc(entry, mutable) for entry in data])
	elif isinstance(data, dict):
		objc = '@{%s}' % ', '.join(['%s: %s' % (data_to_objc(key, mutable), data_to_objc(data[key], mutable)) for key in data])
	if mutable:
		return '[%s mutableCopy]' % objc
	return objc

def data_to_swift(data):
	swift = ''
	if callable(data):
		data = data()
	if data is None:
		return 'nil'
	if isinstance(data, (str, unicode)):
		return '"%s"' % data.replace('"', '\\"')
	elif isinstance(data, bool):
		return 'true' if data else 'false'
	elif isinstance(data, (int, float)):
		return '@%s' % data
	elif isinstance(data, (tuple, list, set)):
		swift = '[%s]' % ', '.join([data_to_swift(entry) for entry in data])
	elif isinstance(data, dict):
		swift = '[%s]' % ', '.join(['%s: %s' % (data_to_swift(key), data_to_swift(data[key])) for key in data])
	return swift
